@@ -1,6 +1,6 @@
 // -*- c++ -*-
 //------------------------------------------------------------------------------
-// $Id: AppearancePrefs.h,v 1.2 2006/10/29 12:51:02 vlg Exp $
+// $Id: AppearancePrefs.h,v 1.5 2007/12/02 00:31:24 vlg Exp $
 //------------------------------------------------------------------------------
 //                            AppearancePrefs.h
 //------------------------------------------------------------------------------
@@ -25,26 +25,11 @@
 #include <gtkmm/checkbutton.h>
 #include <gtkmm/label.h>
 #include <gtkmm/table.h>
-#include <gtkmm/fontselection.h>
 
 class PrefWindow;
 class TextAlignment;
-
-/**-----------------------------------------------------------------------------
-	Compound Font Entry
-
-	+-------+-------+---------------------------+
-	| Label | Entry | Select Font Dialog Button |
-	+-------+-------+---------------------------+
---------------------------------------------------------------------------------
-*/
-class FontEntry 
-{
-public:
-    Gtk::Label*   m_label;
-    Gtk::Entry*   m_entry;		// Font entry
-    Gtk::Button*  m_button;		// Font Dialog selection button
-};
+class FontsSelectorUI;
+class ColorsSelectorUI;
 
 /**-----------------------------------------------------------------------------
  *	General Preferences tab
@@ -55,20 +40,16 @@ class AppearancePrefs : public Gtk::VBox
 public:
     AppearancePrefs (PrefWindow& parent_);
 
-    void on_select_clicked (int idx_);
     void changed_cb        ();
     void load_from_config  ();
     void save_to_config    ();
 
 protected:
-	static const int  FONT_ENTRY_SZ = 5;
+    PrefWindow&  m_pref_window;
 
-    PrefWindow&       m_pref_window;
-
-    FontEntry         m_fentry [FONT_ENTRY_SZ];
-	Gtk::Table*       m_font_table;
-
-	TextAlignment*    m_text_alignment;
+	FontsSelectorUI*   m_fonts_selector;
+	ColorsSelectorUI*  m_colors_selector;
+	TextAlignment*           m_text_alignment;
 };
 
 /**-----------------------------------------------------------------------------
