@@ -2,9 +2,9 @@
 //------------------------------------------------------------------------------
 //                              Deck.cpp
 //------------------------------------------------------------------------------
-// $Id: Deck.cpp,v 1.59 2007/01/15 04:09:08 vlg Exp $
+// $Id: Deck.cpp,v 1.74 2008/02/04 04:02:20 vlg Exp $
 //
-//  Copyright (c) 2004-2006 by Vladislav Grinchenko
+//  Copyright (c) Vladislav Grinchenko
 //
 //  This program is free software; you can redistribute it and/or 
 //  modify it under the terms of the GNU General Public License   
@@ -21,8 +21,8 @@ using std::ifstream;
 #include <glib.h>
 #include <glib/gstdio.h>
 
-#ifdef IS_HILDON
-#  include <hildon-widgets/hildon-system-sound.h>
+#if !defined (WIN32)
+#  include <libgen.h>               // basename(3), dirname(3)
 #endif
 
 #include <assa/CommonUtils.h>
@@ -60,15 +60,21 @@ public:
 Deck::
 Deck (DeckEditStatus status_) 
 	: 
-	m_side_selection        (FRONT),
-	m_status                (status_), 
-	m_needs_renaming        (false),
-	m_snd_enabled           (false)
+	m_side_selection (FRONT),
+	m_status         (status_), 
+	m_needs_renaming (false),
+	m_snd_enabled    (false),
+	m_snd_path_mode  (PATH_ABSOLUTE),
+	m_custom_appearance (false)
 {
 	trace_with_mask("Deck::Deck",GUITRACE|DECK);
 
-	m_active_snd_path = Granule::filename_to_utf8(CONFIG->get_snd_archive_path ());
-	init_appearance ();
+	m_active_snd_path = 
+		Granule::filename_to_utf8 (CONFIG->get_snd_archive_path ());
+
+	m_text_colors_db.init_deck_defaults ();
+	m_fonts_db.init_deck_defaults ();
+	m_appearance.init_deck_defaults ();
 }
 
 /** C'tor for the new deck
@@ -79,42 +85,27 @@ Deck (const string& fname_, DeckEditStatus status_)
 	m_fname  (fname_), 
 	m_desc   ("Created on "),
 	m_status (status_),
-	m_needs_renaming ((status_ == NEW_DECK))
+	m_needs_renaming ((status_ == NEW_DECK)),
+	m_custom_appearance (false)
 {
 	trace_with_mask("Deck::Deck(untitled)",GUITRACE|DECK);
 
-	m_author   = Granule::locale_to_utf8   (CONFIG->get_user_name ());
-	m_active_snd_path = Granule::filename_to_utf8(CONFIG->get_snd_archive_path ());
+	m_author = Granule::locale_to_utf8 (CONFIG->get_user_name ());
+	m_active_snd_path = 
+		Granule::filename_to_utf8 (CONFIG->get_snd_archive_path ());
+
+	m_deck_path = dirname ((char*)fname_.c_str ());
+
+	m_text_colors_db.init_deck_defaults ();
+	m_fonts_db.init_deck_defaults ();
+	m_appearance.init_deck_defaults ();
 
 	{
 		ASSA::TimeVal today = ASSA::TimeVal::gettimeofday ();
 		m_desc += Granule::locale_to_utf8 (today.fmtString ("%c"));
 	}
-	init_appearance ();
-	dump_deck_status ();
-}
 
-void
-Deck::
-init_appearance ()
-{
-	m_custom_app_enabled    = false;
-	m_question_font         = "Sans 18";
-	m_front_x_alignment     = "left";
-	m_front_y_alignment     = "top";
-	m_front_x_padding       = "20";
-	m_front_y_padding       = "20";
-	m_front_justification   = "left";
-	m_back_x_alignment      = "center";
-	m_back_y_alignment      = "center";
-	m_back_x_padding        = "10";
-	m_back_y_padding        = "10";
-	m_back_justification    = "left";
-	m_example_x_alignment   = "left";
-	m_example_y_alignment   = "top";
-	m_example_x_padding     = "12";
-	m_example_y_padding     = "12";
-	m_example_justification = "left";
+	dump_deck_status ();
 }
 
 int
@@ -165,6 +156,7 @@ load (const string& fname_, DeckAction action_)
 	
 	if (action_ == INSTALL_DECK) {
 		m_fname = fname_;
+		m_deck_path = GRANULE->get_dirname (fname_.c_str ());
 	}
 
 	DL((GRAPP,"Parsing the deck ...\n"));
@@ -179,6 +171,7 @@ load (const string& fname_, DeckAction action_)
 		emsg.run ();
 		return -1;
 	}
+
 	/** Setting side selection should come before callback to DECKMGR
 	 */
 	set_side_selection (CONFIG->get_flipside_history (get_name ()));
@@ -188,20 +181,26 @@ load (const string& fname_, DeckAction action_)
 	}
 
 	dump_deck_status ();
-
 	return 0;
 }
 
+/**
+ * This method is where all of the XML parsing actually
+ * takes place.
+ */
 int
 Deck::
 parse_xml_tree (xmlDocPtr parser_, std::string& error_msg_)
 {
 	trace_with_mask("Deck::parse_xml_tree",GUITRACE|DECK);
 
-	char buffer[128];
-	char xpath [64];
-	xmlChar* result;
-	Card* card;
+	char     buffer [1024];
+	char     xpath [64];
+	xmlChar* result = NULL;
+	Card*    card = NULL;
+	
+	std::string deck_dirname = GRANULE->get_dirname (m_fname.c_str ());
+
 	std::ostringstream os;
 	int illegal_count = 0;
 
@@ -226,169 +225,72 @@ parse_xml_tree (xmlDocPtr parser_, std::string& error_msg_)
 		}
 		xmlFree (result);
 	}
-	DL ((APP,"Loaded m_snd_path \"%s\"\n", m_snd_path.c_str ()));
+
+	result = Granule::get_node_by_xpath (parser_, "/deck/sound_path/@relative");
+	if (result)
+	{
+		DL ((APP,"sound_path relative=\"%s\"\n", (char*)result));
+		if (!strcmp ((const char*) result, "yes")) {
+			snd_path_mode (PATH_RELATIVE);
+			m_active_snd_path = Granule::calculate_absolute_path (
+				m_deck_path, m_snd_path);
+		}
+		else {
+			snd_path_mode (PATH_ABSOLUTE);
+		}
+	}
+
+	DL ((APP,"Final m_active_snd_path \"%s\" (%s)\n", 
+		 m_active_snd_path.c_str (), 
+		 (snd_path_mode () == PATH_RELATIVE ? "Relative" : "Absolute")));
+	
+	/** Test the path to see if it is valid.
+	 */
 	test_active_snd_path ();
 
-	/*******************************************
-	 * Load text appearance settings (optional) 
-	 *******************************************/
+	/** Fetch pics_path if present. If absent, assume absolute.
+	 */
+	m_pics_path_mode = PATH_ABSOLUTE;
 
-	result = Granule::get_node_by_xpath (parser_, "/deck/appearance/@enabled");
-	if (result != NULL) {
-		if (!strcmp ((const char*) result, "no")) {
-			m_custom_app_enabled = false;
-			xmlFree (result);
-			goto done_appearance;
+	result = Granule::get_node_by_xpath (parser_, "/deck/pics_path/@relative");
+	if (result) {
+		if (!strcmp ((const char*) result, "yes")) {
+			m_pics_path_mode = PATH_RELATIVE;
 		}
-		m_custom_app_enabled = true;
 		xmlFree (result);
-
-		if (m_custom_app_enabled) {
-			/** question font
-			 */
-			result = Granule::get_node_by_xpath (parser_, 
-									 "/deck/appearance/fonts/@question_font");
-			if (result == NULL) {
-				goto done_appearance;
-			}
-			m_question_font = (const char*) result;
-			xmlFree (result);
-
-			/** front 
-			 */
-			result = Granule::get_node_by_xpath (parser_, 
-									 "/deck/appearance/front_app/@x_alignment");
-			if (result == NULL) {
-				goto done_appearance;
-			}
-			m_front_x_alignment = (const char*) result;
-			xmlFree (result);
-
-			result = Granule::get_node_by_xpath (parser_, 
-									 "/deck/appearance/front_app/@y_alignment");
-			if (result == NULL) {
-				goto done_appearance;
-			}
-			m_front_y_alignment = (const char*) result;
-			xmlFree (result);
-
-			result = Granule::get_node_by_xpath (parser_, 
-									 "/deck/appearance/front_app/@x_padding");
-			if (result == NULL) {
-				goto done_appearance;
-			}
-			m_front_x_padding = (const char*) result;
-			xmlFree (result);
-
-			result = Granule::get_node_by_xpath (parser_, 
-									 "/deck/appearance/front_app/@y_padding");
-			if (result == NULL) {
-				goto done_appearance;
-			}
-			m_front_y_padding = (const char*) result;
-			xmlFree (result);
-
-			result = Granule::get_node_by_xpath (parser_, 
-								 "/deck/appearance/front_app/@justification");
-			if (result == NULL) {
-				goto done_appearance;
-			}
-			m_front_justification = (const char*) result;
-			xmlFree (result);
-
-			/** back 
-			 */
-			result = Granule::get_node_by_xpath (parser_, 
-									 "/deck/appearance/back_app/@x_alignment");
-			if (result == NULL) {
-				goto done_appearance;
-			}
-			m_back_x_alignment = (const char*) result;
-			xmlFree (result);
-
-			result = Granule::get_node_by_xpath (parser_, 
-									 "/deck/appearance/back_app/@y_alignment");
-			if (result == NULL) {
-				goto done_appearance;
-			}
-			m_back_y_alignment = (const char*) result;
-			xmlFree (result);
-
-			result = Granule::get_node_by_xpath (parser_, 
-									 "/deck/appearance/back_app/@x_padding");
-			if (result == NULL) {
-				goto done_appearance;
-			}
-			m_back_x_padding = (const char*) result;
-			xmlFree (result);
-
-			result = Granule::get_node_by_xpath (parser_, 
-									 "/deck/appearance/back_app/@y_padding");
-			if (result == NULL) {
-				goto done_appearance;
-			}
-			m_back_y_padding = (const char*) result;
-			xmlFree (result);
-
-			result = Granule::get_node_by_xpath (parser_, 
-								 "/deck/appearance/back_app/@justification");
-			if (result == NULL) {
-				goto done_appearance;
-			}
-			m_back_justification = (const char*) result;
-			xmlFree (result);
-
-			/** example 
-			 */
-			result = Granule::get_node_by_xpath (parser_, 
-								 "/deck/appearance/example_app/@x_alignment");
-			if (result == NULL) {
-				goto done_appearance;
-			}
-			m_example_x_alignment = (const char*) result;
-			xmlFree (result);
-
-			result = Granule::get_node_by_xpath (parser_, 
-								 "/deck/appearance/example_app/@y_alignment");
-			if (result == NULL) {
-				goto done_appearance;
-			}
-			m_example_y_alignment = (const char*) result;
-			xmlFree (result);
-
-			result = Granule::get_node_by_xpath (parser_, 
-									 "/deck/appearance/example_app/@x_padding");
-			if (result == NULL) {
-				goto done_appearance;
-			}
-			m_example_x_padding = (const char*) result;
-			xmlFree (result);
-
-			result = Granule::get_node_by_xpath (parser_, 
-									 "/deck/appearance/example_app/@y_padding");
-			if (result == NULL) {
-				goto done_appearance;
-			}
-			m_example_y_padding = (const char*) result;
-			xmlFree (result);
-
-			result = Granule::get_node_by_xpath (parser_, 
-								 "/deck/appearance/example_app/@justification");
-			if (result == NULL) {
-				goto done_appearance;
-			}
-			m_example_justification = (const char*) result;
-			xmlFree (result);
-		}
-
 	}
-  done_appearance:
+
+	/**-------------------------------------------------------------------------
+	 *  NOTE: The order is *important* - it must follow granule.dtd -
+	 *        fonts colors ...
+	 **-------------------------------------------------------------------------
+	 */
+	result = Granule::get_node_by_xpath (parser_, "/deck/appearance/@enabled");
+
+	if (result)
+	{
+		/** Optional appearance settings are enabled
+		 */
+		if (!strcmp ((const char*) result, "yes")) {
+			set_custom_appearance (true);
+
+			// Load font settings (optional) 
+			m_fonts_db.load_from_xml_config (parser_, error_msg_);
+
+			// Load text colors settings (optional)
+			m_text_colors_db.load_from_xml_config (parser_, error_msg_);
+
+			// Load text appearance settings (optional) 
+			m_appearance.load_from_xml_config (parser_, error_msg_);
+		}
+		xmlFree (result);
+	}
+
 	dump ();
 
 	/**********************
 	 *   Load the cards   *
 	 **********************/
-
 	for (int cards_counter = 1; ; cards_counter++) 
 	{
 		sprintf (xpath, "/deck/card[%d]", cards_counter);
@@ -399,7 +301,7 @@ parse_xml_tree (xmlDocPtr parser_, std::string& error_msg_)
 		}
 		/** Detected new card
 		 */
-		card = new Card;
+		card = new Card (*this);
 
 		//-----------------------------------------------------
 		sprintf (xpath, "/deck/card[%d]/@id", cards_counter);
@@ -429,10 +331,19 @@ parse_xml_tree (xmlDocPtr parser_, std::string& error_msg_)
 			xmlFree (result);
 		}
 
+		//-----------------------------------------------------
 		sprintf (xpath, "/deck/card[%d]/front/@alt_spelling", cards_counter);
 		result = Granule::get_node_by_xpath (parser_, xpath);
 		if (result != NULL) {
-			card->set_alt_spelling (FRONT, (const char*) result);
+			card->set_alt_spelling (ASF, (const char*) result);
+			xmlFree (result);
+		}
+
+		//-----------------------------------------------------
+		sprintf (xpath, "/deck/card[%d]/front/@image", cards_counter);
+		result = Granule::get_node_by_xpath (parser_, xpath);
+		if (result != NULL) {
+			card->set_image_path ((const char*) result);
 			xmlFree (result);
 		}
 
@@ -444,10 +355,11 @@ parse_xml_tree (xmlDocPtr parser_, std::string& error_msg_)
 			xmlFree (result);
 		}
 
+		//-----------------------------------------------------
 		sprintf (xpath, "/deck/card[%d]/back/@alt_spelling", cards_counter);
 		result = Granule::get_node_by_xpath (parser_, xpath);
 		if (result != NULL) {
-			card->set_alt_spelling (BACK, (const char*) result);
+			card->set_alt_spelling (ASB, (const char*) result);
 			xmlFree (result);
 		}
 
@@ -460,16 +372,19 @@ parse_xml_tree (xmlDocPtr parser_, std::string& error_msg_)
 			card->set_example ((const char*) result);
 			xmlFree (result);
 		}
+
+		sprintf (xpath, "/deck/card[%d]/back_example/@image", cards_counter);
+		result = Granule::get_node_by_xpath (parser_, xpath);
+		if (result != NULL) {
+			card->set_example_image_path ((const char*) result);
+			xmlFree (result);
+		}
+
 		card->mark_clean ();
 		m_cards.push_back (card);
 		card->dump ();
 	}
 
-	if (m_cards.size () == 0) {
-		error_msg_ = "No cards found";
-		return -1;
-	}
-
 	DL((GRAPP,"Processed %d cards\n", m_cards.size ()));
 	return 0;
 }
@@ -808,7 +723,8 @@ split_tokens (const Glib::ustring& instream_,
 			if (instream_[pos_end] == separator_ && !bracket_open) {
 				DL ((CSVPARSER,"copying [%s]\n", 
 					 instream_.substr (pos_start, pos_end-pos_start).c_str ()));
-				tokens_.push_back (instream_.substr (pos_start, pos_end-pos_start));
+				tokens_.push_back (
+					instream_.substr (pos_start, pos_end-pos_start));
 				pos_end++;
 				pos_start = pos_end;
 				DL ((CSVPARSER,"-- next token --\n"));
@@ -846,7 +762,7 @@ add_new_card (Glib::ustring& q_,
 			  long& card_id_,
 			  int& ok_count_)
 {
-	Card* card = new Card;
+	Card* card = new Card (*this);
 	card->set_id (card_id_++);
 
 	/** Trim and replace escape characters as necessary 
@@ -1014,6 +930,8 @@ export_to_csv (const string& file_)
 	sprintf (msg, "Saved %d CSV records", ok_count);
 	Gtk::MessageDialog im (msg, Gtk::MESSAGE_INFO);
 	im.run ();
+
+	return true;
 }
 
 void
@@ -1096,7 +1014,10 @@ save_as (const string& fname_)
 		m_fname = fname_ext;
 		save ();
 		m_fname = tmp;
+
 	}
+	m_deck_path = GRANULE->get_dirname (m_fname.c_str ());
+
 	return 0;
 }
 
@@ -1110,8 +1031,9 @@ save ()
 
 	int ret = 0;
 	xmlTextWriterPtr writer;
-	Glib::ustring alt_spell;
-	cardlist_const_iterator citer;
+	Glib::ustring tmp;
+
+	DL ((GRAPP,"Saving Deck as \"%s\"\n", m_fname.c_str ()));
 
 	g_remove (m_fname.c_str ());
 
@@ -1127,11 +1049,12 @@ save ()
 		return -1;
 	}
 
-	/** We have to disable indent prior to StartDTD call and 
-		enable it back again before EndDTD call to generate expected
-		indentation of XML file. Oddly enough, xmlTextWriterWriteDTD()
-		doesn't do the right thing itself.
-	*/
+	/** 
+	 * We have to disable indent prior to StartDTD call and 
+	 * enable it back again before EndDTD call to generate expected
+	 * indentation of XML file. Oddly enough, xmlTextWriterWriteDTD()
+	 * doesn't do the right thing itself.
+	 */
 	xmlTextWriterSetIndent (writer, 0);
 	xmlTextWriterStartDTD  (writer, BAD_CAST "deck", NULL, BAD_CAST dtd_url);
 	xmlTextWriterSetIndent (writer, 1);
@@ -1144,110 +1067,207 @@ save ()
 									 BAD_CAST m_author.c_str ());
 	ret = xmlTextWriterWriteElement (writer, BAD_CAST "description", 
 									 BAD_CAST m_desc.c_str ());
-	ret = xmlTextWriterWriteElement (writer, BAD_CAST "sound_path", 
-						 BAD_CAST Granule::locale_to_utf8 (m_snd_path).c_str ());
 
-    //--------------------------------------------------------
+	/** If Sound Path is relative, then store only relative
+	 *  portion.
+	 */
+	ret = xmlTextWriterStartElement (writer, BAD_CAST "sound_path");
+
+	if (m_snd_path_mode == PATH_RELATIVE) 
+	{
+		ret = xmlTextWriterWriteAttribute (writer, 
+										   BAD_CAST "relative", BAD_CAST "yes");
+	}
+	else {
+		ret = xmlTextWriterWriteAttribute (writer, 
+										   BAD_CAST "relative", BAD_CAST "no");
+
+	}
+
+	if (!m_snd_path.empty ()) {
+		ret = xmlTextWriterWriteString (writer, 
+					 BAD_CAST Granule::locale_to_utf8 (m_snd_path).c_str ());
+	}
+
+	ret = xmlTextWriterEndElement (writer);	// @sound_path
+
+	/** Record (optional) pics path relativeness.
+	 */
+	ret = xmlTextWriterStartElement (writer, BAD_CAST "pics_path");
+	ret = xmlTextWriterWriteAttribute (writer, 
+									   BAD_CAST "relative", 
+									   pics_path_mode () == PATH_RELATIVE ?
+									   BAD_CAST "yes" : BAD_CAST "no");
+	ret = xmlTextWriterEndElement (writer);	// @pics_path
+
+    /** Save Appearance settings (text fonts and alignments).
+	 */
 	ret = xmlTextWriterStartElement (writer, BAD_CAST "appearance");
 
-    //--------------------------------------------------------
-	ret = xmlTextWriterWriteAttribute (writer, BAD_CAST "enabled",
-							   BAD_CAST (m_custom_app_enabled ? "yes":"no"));
-    //--------------------------------------------------------
-	if (!m_custom_app_enabled) {
-		ret = xmlTextWriterEndElement (writer);	// @appearance
-		goto skip;
+	if (with_custom_appearance ()) 
+	{
+		ret = xmlTextWriterWriteAttribute (writer, 
+										   BAD_CAST "enabled", BAD_CAST "yes");
+		m_fonts_db.save_to_xml_config (writer);
+		m_text_colors_db.save_to_xml_config (writer);
+		m_appearance.save_to_xml_config (writer);
+	}
+	else {
+		ret = xmlTextWriterWriteAttribute (writer, 
+										   BAD_CAST "enabled", BAD_CAST "no");
 	}
-    //--------------------------------------------------------
-	// Continue with Appearance attributes
-	//
-	ret = xmlTextWriterStartElement (writer, BAD_CAST "fonts");
-	ret = xmlTextWriterWriteAttribute (writer, BAD_CAST "question_font",
-									   BAD_CAST m_question_font.c_str ());
-	ret = xmlTextWriterEndElement (writer);	// @fonts
-    //--------------------------------------------------------
-	ret = xmlTextWriterStartElement (writer, BAD_CAST "front_app");
-	ret = xmlTextWriterWriteAttribute (writer, BAD_CAST "x_alignment",
-									   BAD_CAST m_front_x_alignment.c_str ());
-	ret = xmlTextWriterWriteAttribute (writer, BAD_CAST "y_alignment",
-									   BAD_CAST m_front_y_alignment.c_str ());
-	ret = xmlTextWriterWriteAttribute (writer, BAD_CAST "x_padding",
-									   BAD_CAST m_front_x_padding.c_str ());
-	ret = xmlTextWriterWriteAttribute (writer, BAD_CAST "y_padding",
-									   BAD_CAST m_front_y_padding.c_str ());
-	ret = xmlTextWriterWriteAttribute (writer, BAD_CAST "justification",
-									   BAD_CAST m_front_justification.c_str ());
-	ret = xmlTextWriterEndElement (writer);	// @front_app
-    //--------------------------------------------------------
-	ret = xmlTextWriterStartElement (writer, BAD_CAST "back_app");
-	ret = xmlTextWriterWriteAttribute (writer, BAD_CAST "x_alignment",
-									   BAD_CAST m_back_x_alignment.c_str ());
-	ret = xmlTextWriterWriteAttribute (writer, BAD_CAST "y_alignment",
-									   BAD_CAST m_back_y_alignment.c_str ());
-	ret = xmlTextWriterWriteAttribute (writer, BAD_CAST "x_padding",
-									   BAD_CAST m_back_x_padding.c_str ());
-	ret = xmlTextWriterWriteAttribute (writer, BAD_CAST "y_padding",
-									   BAD_CAST m_back_y_padding.c_str ());
-	ret = xmlTextWriterWriteAttribute (writer, BAD_CAST "justification",
-									   BAD_CAST m_back_justification.c_str ());
-	ret = xmlTextWriterEndElement (writer);	// @back_app
-    //--------------------------------------------------------
-	ret = xmlTextWriterStartElement (writer, BAD_CAST "example_app");
-	ret = xmlTextWriterWriteAttribute (writer, BAD_CAST "x_alignment",
-									   BAD_CAST m_example_x_alignment.c_str ());
-	ret = xmlTextWriterWriteAttribute (writer, BAD_CAST "y_alignment",
-									   BAD_CAST m_example_y_alignment.c_str ());
-	ret = xmlTextWriterWriteAttribute (writer, BAD_CAST "x_padding",
-									   BAD_CAST m_example_x_padding.c_str ());
-	ret = xmlTextWriterWriteAttribute (writer, BAD_CAST "y_padding",
-									   BAD_CAST m_example_y_padding.c_str ());
-	ret = xmlTextWriterWriteAttribute (writer, BAD_CAST "justification",
-									 BAD_CAST m_example_justification.c_str());
-	ret = xmlTextWriterEndElement (writer);	// @example_app
-    //--------------------------------------------------------
 
 	ret = xmlTextWriterEndElement (writer);	// @appearance
-    //--------------------------------------------------------
 
-  skip:
-	citer = m_cards.begin ();
+	/** Save all cards
+	 */
+	cardlist_const_iterator citer = m_cards.begin ();
+
 	while (citer != m_cards.end ()) 
 	{
-		//--------------------------------------------------------
+		/**---------------------------------------------------------------------
+		 * @start card
+		 */
 		ret = xmlTextWriterStartElement (writer, BAD_CAST "card");
-		//--------------------------------------------------------
+
+		/**---------------------------------------------------------------------
+		 */
 		ret = xmlTextWriterWriteAttribute (writer, BAD_CAST "id",
 							   BAD_CAST (*citer)->get_id_str ().c_str ());
+		/**---------------------------------------------------------------------
+		 * @start front
+		 */
+		ret = xmlTextWriterStartElement (writer, BAD_CAST "front");	
 
-		//--------------------------------------------------------
-		ret = xmlTextWriterStartElement (writer, BAD_CAST "front");	// @start
-
-		alt_spell = (*citer)->get_alt_spelling (FRONT);
-		if (alt_spell.length () > 0) {
+		tmp = (*citer)->get_alt_spelling (ASF);
+		if (tmp.length () > 0) {
 			ret = xmlTextWriterWriteAttribute (writer, BAD_CAST "alt_spelling",
-											   BAD_CAST alt_spell.c_str ());
+											   BAD_CAST tmp.c_str ());
 		}
+		
+		/** If path is nonempty I save the path.
+		 *  The pics path mode dictates how I should record it -
+		 *  either relative to the Deck path or absolute. If path is
+		 *  out of compliance, convert it on a fly - the end result
+		 *  should be uniform across the board.
+		 */
+		tmp = (*citer)->get_image_path ();
+
+		if (tmp.length () > 0) 
+		{
+			string path = Granule::filename_from_utf8 (tmp);
+			if (pics_path_mode () == PATH_RELATIVE) 
+			{
+				string relpath;				
+				if (g_path_is_absolute (path.c_str ()))
+				{
+					relpath = Granule::calculate_relative_path (
+						m_deck_path, path);
+					ret = xmlTextWriterWriteAttribute (
+						writer, BAD_CAST "image", BAD_CAST relpath.c_str ());
+				}
+				else {
+					ret = xmlTextWriterWriteAttribute (
+						writer, BAD_CAST "image", BAD_CAST tmp.c_str ());
+				}
+			}
+			else {	/* PATH_ABSOLUTE */
+				string abspath;
+				if (!g_path_is_absolute (path.c_str ()))
+				{
+					abspath = Granule::calculate_absolute_path (
+						m_deck_path, path);
+					ret = xmlTextWriterWriteAttribute (
+						writer, BAD_CAST "image", BAD_CAST abspath.c_str ());
+				}
+				else {
+					ret = xmlTextWriterWriteAttribute (
+						writer, BAD_CAST "image", BAD_CAST tmp.c_str ());
+				}		   
+			}
+		}
+
 		ret = xmlTextWriterWriteString (writer,
 								 BAD_CAST (*citer)->get_question ().c_str ());
-		ret = xmlTextWriterEndElement (writer);	// @end front
 
-		//--------------------------------------------------------
-		ret = xmlTextWriterStartElement (writer, BAD_CAST "back"); // @start
+		ret = xmlTextWriterEndElement (writer);
+		
+		/* @end front
+		**----------------------------------------------------------------------
+		** @start back
+		*/
+		ret = xmlTextWriterStartElement (writer, BAD_CAST "back"); 
 
-		alt_spell = (*citer)->get_alt_spelling (BACK);
-		if (alt_spell.length () > 0) {
+		tmp = (*citer)->get_alt_spelling (ASB);
+		if (tmp.length () > 0) {
 			ret = xmlTextWriterWriteAttribute (writer, BAD_CAST "alt_spelling",
-											   BAD_CAST alt_spell.c_str ());
+											   BAD_CAST tmp.c_str ());
 		}
 		ret = xmlTextWriterWriteString (writer,
 								 BAD_CAST (*citer)->get_answer ().c_str ());
-		ret = xmlTextWriterEndElement (writer);	// @end back
+		ret = xmlTextWriterEndElement (writer);
 
-		//--------------------------------------------------------
-		ret = xmlTextWriterWriteElement (writer, BAD_CAST "back_example",
+		/* @end back
+		**----------------------------------------------------------------------
+		** @start back_example
+		*/
+		ret = xmlTextWriterStartElement (writer, BAD_CAST "back_example"); 
+
+		/** If path is nonempty I save the path.
+		 *  The pics path mode dictates how I should record it -
+		 *  either relative to the Deck path or absolute. If path is
+		 *  out of compliance, convert it on a fly - the end result
+		 *  should be uniform across the board.
+		 */
+		tmp = (*citer)->get_example_image_path ();
+
+		if (tmp.length () > 0) 
+		{
+			string path = Granule::filename_from_utf8 (tmp);
+			if (pics_path_mode () == PATH_RELATIVE) 
+			{
+				string relpath;				
+				if (g_path_is_absolute (path.c_str ()))
+				{
+					relpath = Granule::calculate_relative_path (
+						m_deck_path, path);
+					ret = xmlTextWriterWriteAttribute (
+						writer, BAD_CAST "image", BAD_CAST relpath.c_str ());
+				}
+				else {
+					ret = xmlTextWriterWriteAttribute (
+						writer, BAD_CAST "image", BAD_CAST tmp.c_str ());
+				}
+			}
+			else {	/* PATH_ABSOLUTE */
+				string abspath;
+				if (!g_path_is_absolute (path.c_str ()))
+				{
+					abspath = Granule::calculate_absolute_path (
+						m_deck_path, path);
+					ret = xmlTextWriterWriteAttribute (
+						writer, BAD_CAST "image", BAD_CAST abspath.c_str ());
+				}
+				else {
+					ret = xmlTextWriterWriteAttribute (
+						writer, BAD_CAST "image", BAD_CAST tmp.c_str ());
+				}		   
+			}
+		}
+
+		ret = xmlTextWriterWriteString (writer,
 								 BAD_CAST (*citer)->get_example ().c_str ());
+		ret = xmlTextWriterEndElement (writer);
+
+		/* @end back
+		**----------------------------------------------------------------------
+		*/
 
 		ret = xmlTextWriterEndElement (writer);	// @end card
+		/* 
+		** @end card
+		**----------------------------------------------------------------------
+		*/
 		citer++;
 	}
 
@@ -1261,7 +1281,7 @@ save ()
 	m_status = CLEAN_DECK;
 	m_needs_renaming = false;
 
-	/** clear up all cards
+	/** Clear up all cards
 	 */
 	cardlist_iterator iter = m_cards.begin ();
 	while (iter != m_cards.end ()) {
@@ -1275,20 +1295,20 @@ save ()
 
 void
 Deck::
-get_progress (float& pct_done_, std::string& msg_, int idx_)
+get_progress (float& percent_done_, std::string& msg_, int idx_)
 {
 	trace_with_mask("Deck::get_progress",GUITRACE|DECK);
 
-	DL ((APP,"idx_ = %d\n", idx_));
-
 	std::ostringstream os;
-	os << (idx_ + 1) << G_DIR_SEPARATOR_S << m_cards.size ();
+
+	os << (idx_+1) << "/" << m_cards.size ();
 	msg_ = os.str ();
+
 	if (m_cards.size () <= 1) {
-		pct_done_ = 0;
+		percent_done_ = 0;
 	}
 	else {
-		pct_done_ = idx_ * 1.0 / (m_cards.size () - 1);
+		percent_done_ = (idx_ * 1.0) / (m_cards.size () - 1);
 	}
 }
 
@@ -1307,16 +1327,20 @@ find_card_by_id (long id_)
 }
 
 /** Rules for playable name are as follows:
-
-    1) A word can be preceeded by "to " to indicate that it is a verb.
-	2) All multiword expressions are ignored.
-	3) The WAV file is assumed to be stored in the directory
-       with the name of its first letter, i.e. "granule" will be
-       stored in the directory "/path/to/dictionary/g/", and thus 
-	   the full name would be "/path/to/dictionary/g/granule.wav".
-
-	RETURN: playable name on success; an empty string on error.
-*/
+ *
+ *  <ol>
+ *   <li> A word can be preceeded by "to " to indicate that it is a verb. </li>
+ *
+ *   <li> All multiword expressions are ignored. </li>
+ *
+ *	 <li> The WAV file is assumed to be stored in the directory
+ *     with the name of its first letter, i.e. "granule" will be
+ *     stored in the directory "/path/to/dictionary/g/", and thus 
+ *     the full name would be "/path/to/dictionary/g/granule.wav".
+ *   </li>
+ *
+ *	@return Playable name on success; an empty string on error.
+ */
 void
 Deck::
 play_card (SideSelection side_, Deck::cardlist_iterator& iter_)
@@ -1334,10 +1358,18 @@ play_card (SideSelection side_, Deck::cardlist_iterator& iter_)
 
 	DL ((GRAPP,"play card for side '%s'\n", (side_==FRONT?"Front":"Back")));
 
-	w = Granule::locale_from_utf8 ((*iter_)->get_alt_spelling (side_));
+	w = Granule::locale_from_utf8 (
+		(*iter_)->get_alt_spelling (side_ == FRONT ? ASF : ASB));
+
 	if (w.length () == 0) 
 	{
 		w = Granule::locale_from_utf8 ((*iter_)->get_question ());
+		if (w.length () == 0) {
+			/* conversion failed (say, UTF-8 to ASCII)
+			 */
+			return;
+		}
+
 		ASSA::Utils::split (w.c_str (), v);
 		if (v.size () > 1) 
 		{
@@ -1372,16 +1404,16 @@ play_card (SideSelection side_, Deck::cardlist_iterator& iter_)
 
 	DL((APP,"snd filename=\"%s\"\n", sound_fname.c_str ()));
 
-/**
- * Win32
- * -----
- * If sound player command is an empty string, then use
- * build-in system PlaySound() 
- *   (FIXME: test it if it really works. Looks like you still need
- *           the command arguments format to add the sound extension)
- *      
- * Otherwise, use the external command.
- */
+    /**
+	 * Win32
+	 * -----
+	 * If sound player command is an empty string, then use
+	 * build-in system PlaySound() 
+	 *   (FIXME: test it if it really works. Looks like you still need
+	 *           the command arguments format to add the sound extension)
+	 *      
+	 * Otherwise, use the external command.
+	 */
 	/** Arguments composition common for all
 	 */
 	snprintf (execmdbuf, PATH_MAX, cmd_args.c_str (), sound_fname.c_str ());
@@ -1398,42 +1430,39 @@ play_card (SideSelection side_, Deck::cardlist_iterator& iter_)
 		DL((GRAPP,"win32: PlaySound(\"%s\")\n", execmdbuf));
 		PlaySound (execmdbuf, NULL, SND_FILENAME| SND_ASYNC);
 	}
-/**
- * Hildon
- * ------
- * If sound command is an empty string, use build-in playback
- * (this only works with WAV files and produces really bad quality 
- *  sound).
- *
- * Otherwise, use DPS to play mp3 files with no impact on main CPU
- * with this command:
- *
- *   gst-launch filesrc location=/path/to/your.mp3 ! dspmp3sink
- *
- * 
- */
+    /**
+	 * Hildon
+	 * ------
+	 * If sound command is an empty string, use build-in playback
+	 * (this only works with WAV files and produces really bad quality 
+	 *  sound).
+	 *
+	 * Otherwise, use DPS to play mp3 files with no impact on main CPU
+	 * with this command:
+	 *
+	 *   gst-launch filesrc location=/path/to/your.mp3 ! dspmp3sink
+	 *
+	 * 
+	 */
 #elif defined (IS_HILDON)
 	if (cmd != NULL && strlen (cmd) != 0) 
 	{
 		DL((APP,"hildon: fork_exec(\"%s %s\")\n", cmd, execmdbuf));
 //		ASSA::Fork::fork_exec (cmd, execmdbuf, ASSA::Fork::IGNORE_STATUS);
 		n = ASSA::Fork::fork_exec (cmd, execmdbuf, ASSA::Fork::COLLECT_STATUS);
-		DL((GRAPP,"forx_exec() = %d (0:success, 1:failed, -1:wait(2) failed\n",
+		DL((GRAPP,"fork_exec() = %d (0:success, 1:failed, -1:wait(2) failed\n",
 			n));
 	}
-	else {
-		DL((GRAPP,"hildon: hildon_play_system_sound(\"%s\")\n", execmdbuf));
-		hildon_play_system_sound (execmdbuf);
-	}
-/**
- * POSIX (Desktops)
- * ----------------
- * For now, we squarely rely on external command
- *
- * FIXME: for the future, evaluate gstreamer framework.
- *
- *
- */
+
+    /**
+	 * POSIX (Desktops)
+	 * ----------------
+	 * For now, we squarely rely on external command
+	 *
+	 * FIXME: for the future, evaluate gstreamer framework.
+	 *
+	 *
+	 */
 #else
 	/**	
 	 * NOTE: New Gtk 2.4 FileChooserDialog changes disposition mask for SIGCHLD.
@@ -1460,7 +1489,7 @@ play_card (SideSelection side_, Deck::cardlist_iterator& iter_)
 
 // 	DL ((GRAPP,"play card for side '%s'\n", (side_==FRONT?"Front":"Back")));
 
-// 	w = Granule::locale_from_utf8 ((*iter_)->get_alt_spelling (side_));
+//	w = Granule::locale_from_utf8 ((*iter_)->get_alt_spelling (side_ == FRONT ? ASF : ASB));
 
 // 	if (w.length () == 0) 
 // 	{
@@ -1529,13 +1558,13 @@ play_card (SideSelection side_, Deck::cardlist_iterator& iter_)
 // }
 
 /** 
-	Deck information consists of two parts: the header (More... dialog)
-    and a list of cards. Either could have been modified by the user.
-	In the first case, the status would be MODIFIED_DECK; in the second,
-	however, the status would be CLEAN_DECK. We double check here to 
-	see if any of the cards have been edited (if new card were added 
-	or deleted, the status would be MODIFIED_DECK
-*/
+ *	Deck information consists of two parts: the header (More... dialog)
+ *  and a list of cards. Either could have been modified by the user.
+ *  In the first case, the status would be MODIFIED_DECK; in the second,
+ *  however, the status would be CLEAN_DECK. We double check here to 
+ *  see if any of the cards have been edited (if new card were added 
+ *  or deleted, the status would be MODIFIED_DECK.
+ */
 DeckEditStatus
 Deck::
 get_status ()
@@ -1624,14 +1653,21 @@ set_snd_path (const string& spath_)
 	trace_with_mask("Deck::set_snd_path",GUITRACE|DECK);
 
 	m_snd_path = Granule::locale_from_utf8 (spath_.c_str ());
+
 	DL ((APP,"Testing new snd_path \"%s\"\n", m_snd_path.c_str ()));
 
-	if (m_snd_path.length () > 0) {
-		m_active_snd_path = m_snd_path;
+	if (m_snd_path.length () > 0) 
+	{
+		if (m_snd_path_mode == PATH_RELATIVE) {
+			m_active_snd_path = m_deck_path + ASSA_DIR_SEPARATOR_S + m_snd_path;
+		}
+		else {
+			m_active_snd_path = m_snd_path;			
+		}
 	}
 	else {
 		m_active_snd_path = 
-			Granule::filename_to_utf8(CONFIG->get_snd_archive_path ());
+			Granule::filename_to_utf8 (CONFIG->get_snd_archive_path ());
 	}
 
 	test_active_snd_path ();
@@ -1668,7 +1704,7 @@ test_active_snd_path ()
 }
 
 //------------------------------------------------------------------------------
-// LEFTOVERS FROM HEAVY DEBUGGING IN THE PAST
+// LEFTOVERS FROM HEAVY DEBUGGING OF THE PAST
 //------------------------------------------------------------------------------
 #if 0
 void
