@@ -1,10 +1,10 @@
 // -*- c++ -*-
 //------------------------------------------------------------------------------
-// $Id: DeckPlayer.h,v 1.60 2007/01/14 02:00:13 vlg Exp $
+// $Id: DeckPlayer.h,v 1.74 2008/02/04 04:02:20 vlg Exp $
 //------------------------------------------------------------------------------
 //                            DeckPlayer.h
 //------------------------------------------------------------------------------
-//  Copyright (c) 2004-2005 by Vladislav Grinchenko 
+//  Copyright (c) 2004-2007 Vladislav Grinchenko 
 //
 //  This program is free software; you can redistribute it and/or 
 //  modify it under the terms of the GNU General Public License   
@@ -32,51 +32,20 @@
 #include <gtkmm/table.h>
 
 #ifdef IS_HILDON
-#include <hildon-libsmm.h>
+#include <hildonmm.h>
 #endif
 
 #include "GrappConf.h"
 #include "VDeck.h"
 
-class DeckView;
-class DeckPlayer;
 class Gtk::ProgressBar;
 class Gtk::EventBox;
 
-/**
- * External events such as control buttons click
- * also cause switching between states.
- */
-typedef enum {
-	VC_START,					// Waiting for input
-	VC_ARMED,					// User entered text into VC entry
-	VC_VALIDATED				// Entry text validated
-} VCState;
-
+class DeckView;
+class DeckPlayer;
+class TestLineEntry;
 class VerifyControl;
-
-/**=============================================================================
- *   Class TestLineEntry encapsulates line selection widget
- **=============================================================================
- */
-class TestLineEntry : public Gtk::HBox
-{
-public:
-	TestLineEntry (VerifyControl* parent_);
-
-	int get_value_as_int () const;
-
-private:
-	/** For desktop and Hildon
-	 */
-	Gtk::Adjustment m_adjustment;
-	Gtk::Label      m_label;
-	Gtk::SpinButton m_test_line_button;
-
-	/** For PDA 
-	 */
-	Gtk::Entry      m_entry;
-};
+class ButtonWithImageLabel;
 
 /**=============================================================================
  *   Class SideFlipControl encapsulates the side switch radio buttons
@@ -107,55 +76,6 @@ private:
 	SideSelection    m_side;
 };
 
-/**=============================================================================
- *   Class VerifyControl encapsulates the verification element 
- *   of the DeckPlayer dialog. This element is optionally shown
- *   in the dialog between the notebook and the control toolbar.
- **=============================================================================
- */
-class VerifyControl : public Gtk::Table
-{
-public:
-	VerifyControl (DeckPlayer& parent_);
-
-	void    set_focus_on_entry ();
-
-	void    compare (const Glib::ustring& answer_);
-	bool    matched () const { return m_was_equal; }
-	void    clear   ();
-
-	VCState get_state () const;
-	void    set_state (VCState state_);
-
-	void    on_entry_changed ();
-	void    on_test_line_changed ();
-
-	void    attach (TestLineEntry* tle_) { m_test_line = tle_; }
-	void    repaint () { m_entry->modify_font (CONFIG->get_input_font ()); }
-
-private:
-	static const int LABELS_SZ = 4;
-
-	DeckPlayer&      m_parent;
-
-	Gtk::Label*      m_report;		      // Correct, Incorrect
-	Gtk::EventBox*   m_report_box;        // Box to hold report label
-	Gtk::Entry*      m_entry;
-	Gtk::Button*     m_button;	          // 'Check answer', 'Next'
-	Gtk::Label*      m_label [LABELS_SZ];
-
-	bool             m_was_equal;         // Keeps result of last compare() op.
-	VCState          m_state;
-
-	Gdk::Color       m_red;
-	Gdk::Color       m_green;
-	Gdk::Color       m_white;
-	Gdk::Color       m_black;
-
-	TestLineEntry*   m_test_line;         // Which line to compare for answer.
-	int              m_line_num;          // Number of line to test
-};
-
 /**=============================================================================
  *                              Class DeckPlayer 
  **=============================================================================
@@ -169,9 +89,18 @@ class DeckPlayer : public Gtk::Window
 public:
 	DeckPlayer (VDeck& vdeck_);
 
-	/** Run the *dialog*
+	/** Simulate re-entrant event loop. Also, hide/show parent
+	 *  window and reposition accordingly.
 	 */
-	gint run ();
+	gint run (Gtk::Window& parent_);
+
+	/** We block [x] delete action and reroute it to <Close>
+	 */
+	virtual bool on_delete_event (GdkEventAny* event_) 
+	{
+		on_close_clicked ();
+		return true;
+	}
 
 	/** Called in response to pressing <Close> button
 	 */
@@ -236,6 +165,10 @@ public:
 	 */
 	VDeck* get_deck () const { return &m_deck; }
 
+	/** Is it a virtual deck or the real one.
+	 */
+	bool real_deck () const { return m_is_real_deck; }
+
 	/** Reset iterator/count to reflect new card(s) added to the deck.
 		Also, point iterator to the first card and show it. This method
 		should be called each time the VDeck's count is modified (card is
@@ -243,6 +176,10 @@ public:
 	*/
 	void reset ();
 
+	/** Set expiration light. 
+	 */
+	void set_light (Glib::RefPtr<Gdk::Pixbuf> l_) { m_exp_light->set (l_); }
+
 	/// Reset geometry from Configurator
 	void update_geometry ();
 
@@ -269,6 +206,7 @@ public:
 
 	SideSelection get_selected_side () const { return m_selected_side; }
 
+public:
 	/** Remove all newlines and shrink all whitespace.
 	*/
 	static void shrink_to_single_line (Glib::ustring& txt_);
@@ -285,11 +223,15 @@ private:
 	void adjust_count    ();	// Adjust count label: current/total
 	void set_sensitivity ();    // De-sensetize inappropriate controls
 	void set_win_name    ();    // Set tile name
+	void clear_selection ();    // Clear any selected text (annoying)
+
+	void reset_text_fonts ();
+	void reset_text_colors ();
 
-	Pango::FontDescription question_font ();
 	Gtk::AlignmentEnum x_alignment_enum   (SideSelection s_);
 	Gtk::AlignmentEnum y_alignment_enum   (SideSelection s_);
 	Gtk::Justification justification_enum (SideSelection s_);
+
 	int x_padding_val (SideSelection s_);
 	int y_padding_val (SideSelection s_);
 
@@ -323,30 +265,55 @@ private:
 	Gtk::VBox* get_vbox () { return m_vbox; }
 
 private:
+	static const string RATIO_TITLE;
+	static const string REVIEW_TITLE;
+	static const string RIGHT_WRONG_TITLE;
 
 #ifdef IS_HILDON
 	bool              m_hildon_fullscreen;
 #endif
 
-	Gtk::Button*      m_close_button;
-	Gtk::Button*      m_shuffle_button;
-	Gtk::Button*      m_edit_button;         // View/Edit a Deck
-	Gtk::Button*      m_vcard_edit_button;   // Edit a Card
+	ButtonWithImageLabel* m_close_button;
+	ButtonWithImageLabel* m_shuffle_button;
+	ButtonWithImageLabel* m_edit_button;         // View/Edit a Deck
+	ButtonWithImageLabel* m_vcard_edit_button;   // Edit a Card
+
 	int               m_match_front_row;     // Which row to select for answer
-	Gtk::Label        m_count_label;
+
+	Gtk::Label        m_count_label;         // Reviewed/Total
+	Gtk::Label        m_success_ratio_label; // Success ratio
+	Gtk::Label        m_right_wrong_label;   // Right/Wrong answers
+	Gtk::Image*       m_exp_light;           // Expiration light
+	Glib::RefPtr<Gdk::Pixbuf> m_sched_grey;  // Default color of exp. light
+
+	SideFlipControl*  m_sf_control;
 	SideSelection     m_selected_side;
+
 	Gtk::Notebook*    m_notebook;
 	Gtk::ProgressBar* m_play_progress;
+
 	Gtk::Label        m_front;
 	Gtk::Label        m_back;
 	Gtk::Label        m_example;
+
+	Gtk::Image*       m_front_image; // (optional) image for the Front side
+	Glib::RefPtr<Gdk::Pixbuf> m_front_img_pixbuf;
+
+	Gtk::Image*       m_example_image; // (optional) image for the Front side
+	Glib::RefPtr<Gdk::Pixbuf> m_example_img_pixbuf;
+
 	TestLineEntry*    m_test_line_entry;
 
 	Gtk::VBox*        m_question_box; /* VBox that holds the front of a card */
 	int               m_question_box_width;             /* last resize width */
+	Gtk::EventBox*    m_question_evbox;
+
 	Gtk::VBox*        m_answer_box;   /* VBox that holds the back of a card  */
 	int               m_answer_box_width;               /* last resize width */
 
+	Gtk::EventBox*    m_sep_evbox;
+	Gtk::Viewport*    m_page_viewport;
+
 	VerifyControl*    m_verify_control;	// optional answer verification entry 
 
 	Gtk::Widget*      m_toolbar_go_first;
@@ -366,10 +333,6 @@ private:
 	Gtk::Button*      m_hs_button_vc;
 	Gtk::Button*      m_hs_button_ntb;
 
-	Gdk::Color        m_text_fgcolor;
-	Gdk::Color        m_text_bgcolor;
-	Gdk::Color        m_sep_color;
-
 	VDeck&            m_deck;
 	VDeck::cardlist_iterator m_deck_iter; // Position in the Deck
 
@@ -382,18 +345,15 @@ private:
 	bool m_tab_activated;		// Leave Tab+Spacebar/Enter event to Dialog
 	bool m_initialized;	        // Has the object been fully constructed?
 
-	Gtk::VBox*        m_vbox;
-	Gtk::HBox*        m_navigation_controls_hbox;
+	Gtk::VBox*     m_vbox;
+	Gtk::HBox*     m_navigation_controls_hbox;
+	Gtk::Toolbar*  m_ctrl_toolbar;
+	Gtk::EventBox* m_nav_ctrl_evbox;
 };
 
-inline void
-DeckPlayer::
-flip_card ()
-{
-	m_notebook->set_current_page (m_notebook->get_current_page () ? 0 : 1);
-	m_verify_control->set_focus_on_entry ();
-}
-
+//------------------------------------------------------------------------------
+// Inline functions
+//------------------------------------------------------------------------------
 inline void
 DeckPlayer::
 clear_text ()
@@ -401,6 +361,7 @@ clear_text ()
 	m_front.  set_text ("");
 	m_back.   set_text ("");
 	m_example.set_text ("");
+	m_front_image->hide ();
 }
 
 /*------------------------------------------------------------
