@@ -1,6 +1,6 @@
 // -*- c++ -*-
 //------------------------------------------------------------------------------
-// $Id: DeckView.cpp,v 1.54 2007/01/15 04:09:08 vlg Exp $
+// $Id: DeckView.cpp,v 1.66 2008/02/04 04:02:20 vlg Exp $
 //------------------------------------------------------------------------------
 //                            DeckView.cpp
 //------------------------------------------------------------------------------
@@ -24,6 +24,7 @@
 #include <assa/AutoPtr.h>
 
 #include "Granule-main.h"
+#include "GrappConf.h"
 #include "Granule.h"
 #include "MainWindow.h"
 #include "Card.h"
@@ -33,6 +34,8 @@
 #include "DeckView.h"
 #include "DeckInfo.h"
 #include "ButtonWithImageLabel.h"
+#include "CardAutoFill.h"
+
 #include "Intern.h"
 
 using sigc::bind;
@@ -62,74 +65,11 @@ DeckView (VDeck& deck_, int selected_row_)
 {
 	trace_with_mask("DeckView::DeckView", GUITRACE);
 
-	m_actgroup = Gtk::ActionGroup::create ();
-
-	m_actgroup->add (Gtk::Action::create ("Add", 
-										  Gtk::Stock::ADD,
-										  "Add",
-										  "Add new card"),
-					 mem_fun (*this, &DeckView::on_add_clicked));
-
-	m_actgroup->add (Gtk::Action::create ("Edit", 
-										  Gtk::Stock::DND,
-										  "Edit",
-										  "Edit selected card"),
-					 mem_fun (*this, &DeckView::on_edit_clicked));
-
-	m_actgroup->add (Gtk::Action::create ("Delete", 
-#ifdef WIN32
-										  Gtk::Stock::CUT,
-#else
-										  Gtk::Stock::DELETE,
-#endif
-										  "Delete",
-										  "Delete selected card"),
-					 mem_fun (*this, &DeckView::on_delete_clicked));
-
-	m_actgroup->add (Gtk::Action::create ("Go_Up", 
-										  Gtk::Stock::GO_UP,
-										  "Move Up",
-										  "Move selecte card up"),
-					 mem_fun (*this, &DeckView::on_card_up_clicked));
-
-	m_actgroup->add (Gtk::Action::create ("Go_Down", 
-										  Gtk::Stock::GO_DOWN,
-										  "Move Down",
-										  "Move selected card down"),
-					 mem_fun (*this, &DeckView::on_card_down_clicked));
-
-	m_actgroup->add (Gtk::Action::create ("Add_Box1", 
-										  Gtk::Stock::UNDO,
-										  "Add To Box1",
-										  "Add selected card to Box1"),
-					 mem_fun (*this, &DeckView::on_add_card_to_cardbox));
-
-	m_actgroup->add (Gtk::Action::create ("Deck_Info", 
-										  Gtk::Stock::DIALOG_INFO,
-										  "Deck Info",
-										  "Deck Information"),
-					 mem_fun (*this, &DeckView::on_more_clicked));
-
-	m_actgroup->add (Gtk::Action::create ("Save_Deck", 
-										  Gtk::Stock::SAVE,
-										  "Save",
-										  "Save deck"),
-					 mem_fun (*this, &DeckView::on_save_deck_clicked));
-
-    m_uimanager = Gtk::UIManager::create ();
-    m_uimanager->insert_action_group (m_actgroup);
-
-#ifndef IS_HILDON
-    add_accel_group (m_uimanager->get_accel_group ());
-#endif
-
-
 	/********************************
 	 * Setup dialog attributes
 	 ********************************/
 	set_title("Deck View");
 
-#ifdef IS_HILDON
 	/**
 	 * Not a dialog any more - it is a Bin (AppView) instead.
 	 * We handle packaging ourselves and mute all attributes of a 
@@ -137,10 +77,11 @@ DeckView (VDeck& deck_, int selected_row_)
 	 */
     m_vbox = manage (new Gtk::VBox (false, 2));
 	add (*m_vbox);
+
+#ifdef IS_HILDON
 	set_size_request (672, 396);
 #else
 	set_modal (true);
-//	set_position (Gtk::WIN_POS_CENTER_ALWAYS);
 #ifdef IS_PDA
 	set_resizable(false);
 	Gdk::Geometry dp_geometry =	{ 240, 320,	240, 320, 240, 320,	-1, -1,	-1, -1};
@@ -150,46 +91,133 @@ DeckView (VDeck& deck_, int selected_row_)
 	set_resizable(true);
 	set_size_request (400, 370);	// width; height 
 	set_border_width (2);
-	set_transient_for (*MAINWIN);
 #endif // (!IS_PDA)
 #endif // (!IS_HILDON)
 
+	/** Add <Close> button to the Gtk::Dialog
+	 */
+	m_close_button = manage (new ButtonWithImageLabel (
+								 _("<u>C</u>lose"), Gtk::Stock::CLOSE));
+	m_close_button->set_alt_accelerator_key (GDK_c, get_accel_group ());
+	m_close_button->set_flags  (Gtk::CAN_FOCUS);
+	m_close_button->set_relief (Gtk::RELIEF_NORMAL);
+
+	m_close_button->signal_clicked ().connect (
+		mem_fun (*this, &DeckView::on_close_clicked));
+
+	/** 
+	 * For the top edit buttons bar, create an ActionGroup.
+	 */
+	m_actgroup = Gtk::ActionGroup::create ("Actions");
+
+	m_actgroup->add (Gtk::Action::create ("Add", 
+										  Gtk::Stock::ADD,
+										  _("_Add"),
+										  "Add new card"),
+					 mem_fun (*this, &DeckView::on_add_clicked));
+
+	m_actgroup->add (Gtk::Action::create ("Edit", 
+										  Gtk::Stock::DND,
+										  _("_Edit"),
+										  "Edit selected card"),
+					 mem_fun (*this, &DeckView::on_edit_clicked));
+
+	m_actgroup->add (Gtk::Action::create ("AutoFill", 
+										  Gtk::Stock::CONVERT,
+										  _("Auto _Fill"),
+										  "AutoFill Card fields"),
+					 mem_fun (*this, &DeckView::on_autofill_clicked));
+
+	m_actgroup->add (Gtk::Action::create ("Delete", 
+#ifdef WIN32
+										  Gtk::Stock::CUT,
+#else
+										  Gtk::Stock::DELETE,
+#endif
+										  _("_Delete"),
+										  "Delete selected card"),
+					 mem_fun (*this, &DeckView::on_delete_clicked));
+
+	m_actgroup->add (Gtk::Action::create ("Go_Up", 
+										  Gtk::Stock::GO_UP,
+										  _("Move Up"),
+										  "Move selecte card up"),
+					 mem_fun (*this, &DeckView::on_card_up_clicked));
+
+	m_actgroup->add (Gtk::Action::create ("Go_Down", 
+										  Gtk::Stock::GO_DOWN,
+										  _("Move Down"),
+										  "Move selected card down"),
+					 mem_fun (*this, &DeckView::on_card_down_clicked));
+
+	m_actgroup->add (Gtk::Action::create ("Add_Box1", 
+										  Gtk::Stock::UNDO,
+										  _("Add To CardBox 1"),
+										  "Add selected card to Box1"),
+					 mem_fun (*this, &DeckView::on_add_card_to_cardbox));
+
+	m_actgroup->add (Gtk::Action::create ("Deck_Info", 
+										  Gtk::Stock::PREFERENCES,
+										  _("_Preferences"),
+										  "Deck preferences"),
+					 mem_fun (*this, &DeckView::on_more_clicked));
+
+	m_actgroup->add (Gtk::Action::create ("Save_Deck", 
+										  Gtk::Stock::SAVE,
+										  _("_Save"),
+										  "Save deck"),
+					 mem_fun (*this, &DeckView::on_save_deck_clicked));
+
+    m_uimanager = Gtk::UIManager::create ();
+    m_uimanager->insert_action_group (m_actgroup);
+
+#ifndef IS_HILDON
+	Gtk::Window* this_win = dynamic_cast<Gtk::Window*> (this);
+    this_win->add_accel_group (m_uimanager->get_accel_group ());
+#endif
+
 	/********************************
 	 * Setup toolbar                
 	 ********************************/
+	Glib::ustring ui_info =
+		"<ui>"
+		"  <toolbar name='DV_Controls'>"
+		"     <toolitem action='Add'/>"
+		"     <toolitem action='Edit'/>"
+		"     <toolitem action='AutoFill'/>"
+		"     <toolitem action='Delete'/>"
+		"     <toolitem action='Go_Up'/>"
+		"     <toolitem action='Go_Down'/>"
+		"     <toolitem action='Add_Box1'/>"
+		"     <toolitem action='Deck_Info'/>"
+		"     <toolitem action='Save_Deck'/>"
+		"  </toolbar>"
+		"</ui>";
+
+#ifdef GLIBMM_EXCEPTIONS_ENABLED
 	try {
-		Glib::ustring ui_info =
-			"<ui>"
-			"  <toolbar name='DV_Controls'>"
-			"     <toolitem action='Add'/>"
-			"     <toolitem action='Edit'/>"
-			"     <toolitem action='Delete'/>"
-			"     <toolitem action='Go_Up'/>"
-			"     <toolitem action='Go_Down'/>"
-			"     <toolitem action='Add_Box1'/>"
-			"     <toolitem action='Deck_Info'/>"
-			"     <toolitem action='Save_Deck'/>"
-			"  </toolbar>"
-			"</ui>";
-#ifdef GLIBMM_EXCEPTOINS_ENABLED
 		m_uimanager->add_ui_from_string (ui_info);
-#else
-		std::auto_ptr<Glib::Error> err;
-		m_uimanager->add_ui_from_string (ui_info, err);
-#endif
     }
     catch (const Glib::Error& ex_) {
-		DL((GRAPP,"Building menues failed (%s)\n", ex_.what ().c_str ()));
+		DL((GRAPP,"Building menus failed (%s)\n", ex_.what ().c_str ()));
     }
-		
-	m_add_button          = m_uimanager->get_widget ("/DV_Controls/Add");
-	m_edit_button         = m_uimanager->get_widget ("/DV_Controls/Edit");
-	m_delete_button       = m_uimanager->get_widget ("/DV_Controls/Delete");
-	m_card_up_button      = m_uimanager->get_widget ("/DV_Controls/Go_Up");
-	m_card_down_button    = m_uimanager->get_widget ("/DV_Controls/Go_Down");
-	m_add_card_to_cardbox = m_uimanager->get_widget ("/DV_Controls/Add_Box1");
-	m_more_button         = m_uimanager->get_widget ("/DV_Controls/Deck_Info");
-	m_save_deck_button    = m_uimanager->get_widget ("/DV_Controls/Save_Deck");
+#else
+	std::auto_ptr<Glib::Error> err;
+	m_uimanager->add_ui_from_string (ui_info, err);
+	if (err.get ()) {
+		DL((GRAPP,"Building menus failed (%s)\n", err->what ().c_str ()));
+	}
+#endif
+
+	m_add_button       = (m_uimanager->get_widget ("/DV_Controls/Add"));
+	m_edit_button      = (m_uimanager->get_widget ("/DV_Controls/Edit"));
+	m_autofill_button  = (m_uimanager->get_widget ("/DV_Controls/AutoFill"));
+	m_delete_button    = (m_uimanager->get_widget ("/DV_Controls/Delete"));
+	m_card_up_button   = (m_uimanager->get_widget ("/DV_Controls/Go_Up"));
+	m_card_down_button = (m_uimanager->get_widget ("/DV_Controls/Go_Down"));
+	m_add_card_to_cardbox = (m_uimanager->get_widget ("/DV_Controls/Add_Box1"));
+	m_more_button      = (m_uimanager->get_widget ("/DV_Controls/Deck_Info"));
+	m_save_deck_button = (m_uimanager->get_widget ("/DV_Controls/Save_Deck"));
 
     Gtk::Widget* controls = m_uimanager->get_widget ("/DV_Controls");
     Gtk::Toolbar* ctrl_toolbar = dynamic_cast<Gtk::Toolbar*> (controls);
@@ -199,7 +227,7 @@ DeckView (VDeck& deck_, int selected_row_)
     ctrl_toolbar->set_orientation (Gtk::ORIENTATION_HORIZONTAL);
     ctrl_toolbar->set_toolbar_style (Gtk::TOOLBAR_ICONS);
 
-	get_vbox ()->pack_start (*ctrl_toolbar, Gtk::PACK_SHRINK, 0);
+	m_vbox->pack_start (*ctrl_toolbar, Gtk::PACK_SHRINK, 0);
 
 	/**************************
 	 * List View 
@@ -242,17 +270,21 @@ DeckView (VDeck& deck_, int selected_row_)
 
 	/******************************
 	 * Create TreeView
-	 ******************************/
+	 ******************************
+	 */
 	DL((GRAPP,"Create TreeView\n"));
 	m_tree_view.set_model (m_list_store_ref);
 	m_tree_view.set_rules_hint ();
+	m_tree_view.set_headers_clickable (true);
+	m_tree_view.set_headers_visible (true);
 
 	Gtk::TreeView::Column* column;
 	Gtk::CellRendererText* renderer;
 
 	/******************************
 	 * Front: Column
-	 ******************************/
+	 ******************************
+	 */
 	column = Gtk::manage (new Gtk::TreeView::Column ("Front"));
 	m_tree_view.append_column (*column);
 	renderer = Gtk::manage (new Gtk::CellRendererText);
@@ -275,7 +307,8 @@ DeckView (VDeck& deck_, int selected_row_)
 
 	/******************************
 	 * Enable column sorting
-	 ******************************/
+	 ******************************
+	 */
 	column->set_sort_column (m_columns.m_front_row);
 	column->set_sort_indicator (true);
 	column->signal_clicked ().connect (
@@ -283,12 +316,14 @@ DeckView (VDeck& deck_, int selected_row_)
 
 	/******************************
 	 * Back: Column
-	 ******************************/
+	 ******************************
+	 */
 	column = Gtk::manage (new Gtk::TreeView::Column ("Back"));
 	m_tree_view.append_column (*column);
 	renderer = Gtk::manage (new Gtk::CellRendererText);
 
 	column->pack_start (*renderer, false);
+
 #ifdef GLIBMM_PROPERTIES_ENABLED
 	column->add_attribute (renderer->property_markup (), 
 						   m_columns.m_back_row);
@@ -301,7 +336,8 @@ DeckView (VDeck& deck_, int selected_row_)
 
 	/****************************
 	 * Enable column sorting
-	 ****************************/
+	 ****************************
+	 */
 	column->set_sort_column (m_columns.m_back_row);
 	column->set_sort_indicator (true);
 	column->signal_clicked ().connect (
@@ -312,24 +348,35 @@ DeckView (VDeck& deck_, int selected_row_)
 	m_tree_sel_ref->signal_changed ().connect (
 		mem_fun (*this, &DeckView::on_card_selected));
 
-	m_tree_view.add_events (Gdk::BUTTON_PRESS_MASK);
+	/**
+	 * If the second argument is 'false', it tells Gtk+ to call 
+	 * on_key_pressed() BEFORE keypress event handling. 
+	 * Returning 'true' from on_key_pressed() stops further event processing.
+	 */
+	m_tree_view.add_events (Gdk::BUTTON_PRESS_MASK|Gdk::KEY_PRESS_MASK);
+
 	m_tree_view.signal_button_press_event ().connect (
-		sigc::mem_fun (*this, &DeckView::on_mouse_click), false);
+		sigc::mem_fun (*this, &DeckView::on_button_press_event), false);
+
+	m_tree_view.signal_key_press_event ().connect (
+		sigc::mem_fun (*this, &DeckView::on_key_tapped), false);
 
 	m_scrollwin->add (m_tree_view);
 
 	/*****************************
 	 * Pack the cards list
-	 *****************************/
+	 *****************************
+	 */
 	Gtk::HBox* h_viewbox = manage (new Gtk::HBox(false, 0));
 	h_viewbox->pack_start (*m_scrollwin, Gtk::PACK_EXPAND_WIDGET, 0);
-	get_vbox ()->pack_start (*h_viewbox);
+	m_vbox->pack_start (*h_viewbox);
 
 	/***********************************
 	 *  h_infobox holds:
 	 *  -------------------------
 	 *      [Total:] [XXX cards] 
-	 ***********************************/
+	 ***********************************
+	 */
 	Gtk::Label* label_total = manage (new Gtk::Label ("Total: "));
 	Gtk::HBox* h_infobox    = manage (new Gtk::HBox(false, 0));
 
@@ -339,29 +386,13 @@ DeckView (VDeck& deck_, int selected_row_)
 	h_infobox->pack_start (*label_total,   Gtk::PACK_SHRINK, 9);
 	h_infobox->pack_start (*m_label_count, Gtk::PACK_SHRINK, 0);
 
-	get_vbox ()->pack_start (*h_infobox, Gtk::PACK_SHRINK, 4);
+	m_vbox->pack_start (*h_infobox, Gtk::PACK_SHRINK, 4);
 
-	/** <Close> button
-	 */
-#ifdef IS_HILDON  // Handle the button ourselves
-	m_close_button = manage (new ButtonWithImageLabel (Gtk::Stock::CLOSE, 
-													   "Close"));
-#else  // Add button to the Gtk::Dialog
-	m_close_button = add_button (Gtk::StockID ("gtk-close"), 
-								 Gtk::RESPONSE_CLOSE );
-#endif
-	m_close_button->set_flags  (Gtk::CAN_FOCUS);
-	m_close_button->set_relief (Gtk::RELIEF_NORMAL);
-
-	m_close_button->signal_clicked ().connect (
-		mem_fun (*this, &DeckView::on_close_clicked));
-
-#ifdef IS_HILDON
 	/**
 	 * Add separator and [Close] button in the ButtonBox
 	 */
 	Gtk::HSeparator* hseparator = manage (new Gtk::HSeparator);
-	get_vbox ()->pack_start (*hseparator, false, true, 0);
+	m_vbox->pack_start (*hseparator, false, true, 0);
 
 	Gtk::HButtonBox*  hbuttonbox = manage (new Gtk::HButtonBox);
 	hbuttonbox->set_spacing (6);
@@ -371,24 +402,26 @@ DeckView (VDeck& deck_, int selected_row_)
 	hbuttonbox->pack_start (*expander, Gtk::PACK_EXPAND_WIDGET);
 	hbuttonbox->pack_end (*m_close_button, Gtk::PACK_SHRINK);
 
-	get_vbox ()->pack_start (*hbuttonbox, Gtk::PACK_SHRINK);
-
-#endif
+	m_vbox->pack_start (*hbuttonbox, Gtk::PACK_SHRINK);
 
 	set_cards_count ();
 	show_all ();
 
-	/** Select the same row (card) that the DeckPlayer has selected,
-		and scroll to it. 
-
-		NOTE: There are two stunning points worth noting here.
-		First, upon executing show_all(), Gtkmm internals set the
-		selection to row # 0 and then emmit *changed* signal!
-		Bottom line - if you want to select a row in c'tor, do 
-		your selection AFTER show_all(). 
-		Secondly, select (iter) doesn't work, but select (row) does.
+	/** 
+	 * Select the same row (card) that the DeckPlayer has selected,
+	 * and scroll to it. 
+	 *
+	 * NOTE: There are two stunning points worth noting here.
+	 *
+	 *       First, upon executing show_all(), Gtkmm internals set the
+	 *       selection to row # 0 and then emmit *changed* signal!
+	 *       Bottom line - if you want to select a row in c'tor, do 
+	 *       your selection AFTER show_all(). 
+	 *
+	 *       Secondly, select (iter) doesn't work, but select (row) does.
 	 */
 	DL((APP,"m_row_idx = %d\n", m_row_idx));
+
 	if (m_row_idx >= 0) {
 		m_row_idx = selected_row_;
 		children_t children = m_list_store_ref->children();
@@ -403,6 +436,7 @@ DeckView (VDeck& deck_, int selected_row_)
 	set_icon_from_file (GRAPPDATDIR "/pixmaps/deckview_32x32.png");
 #endif
 
+	m_tree_view.grab_focus ();
 }
 
 DeckView::
@@ -425,18 +459,18 @@ set_cards_count ()
 //                              Callbacks
 //------------------------------------------------------------------------------
 
-//#ifdef IS_HILDON
 gint
 DeckView::
-run ()
+run (Gtk::Window& parent_)
 {
     trace_with_mask("DeckView::run",GUITRACE);
 
-	Gtk::Main::run ();
+	GRANULE->move_child_to_parent_position (parent_, *this);
+	gtk_main ();
+	GRANULE->move_parent_to_child_position (parent_, *this);
 
 	return (Gtk::RESPONSE_NONE);
 }
-//#endif
 
 void
 DeckView::
@@ -444,9 +478,7 @@ on_close_clicked ()
 {
 	trace_with_mask("DeckView::on_close_clicked", GUITRACE);
 
-	hide ();
-	DL ((GRAPP,"Call Main::quit()\n"));
-	Gtk::Main::quit ();
+	gtk_main_quit ();
 }
 
 void
@@ -475,8 +507,7 @@ on_card_selected ()
 	if (iter == pathlist.end ()) {
 		return;
 	}
-// Depricated
-//	m_row_idx = *((*iter).get_indices ().begin ());
+
 	m_row_idx = *((*iter).begin ());
 	DL((GRAPP,"Row selected : %d\n", m_row_idx));
 }
@@ -487,6 +518,7 @@ add_new_card (Card* card_)
 {
 	trace_with_mask("DeckView::add_new_card", GUITRACE);
 
+	DL ((GRAPP,"Adding new card id = %d\n", card_->get_id ()));
 	m_deck.push_back (card_);
 
 	Gtk::TreeModel::Row row = *m_list_store_ref->append ();
@@ -510,17 +542,20 @@ on_add_clicked ()
 {
 	trace_with_mask("DeckView::on_add_clicked", GUITRACE);
 
-	hide ();
+	/** It is safe to cast VDeck& to Deck& here
+	 *  because we can only perform add/edit operation on
+	 *  a real Deck.
+	 */
+	CardView cview (new Card (dynamic_cast<Deck&>(m_deck)));
 
-	CardView cview (new Card);
-
-	int ret = cview.run (*this);
-	DL((GRAPP,"cview.run () = %d\n", ret));
+	int ret = cview.run_in_add_mode (*this, CONFIG->autofill_db ());
 
 	if (ret == Gtk::RESPONSE_OK) {
 		DL((GRAPP,"Card accepted\n"));
+		return;
 	}
-	else if (ret == Gtk::RESPONSE_CANCEL) {
+	
+	if (ret == Gtk::RESPONSE_CANCEL) {
 		DL((GRAPP,"Card cancelled\n"));
 	}
 	else if (ret == Gtk::RESPONSE_REJECT) {
@@ -530,7 +565,6 @@ on_add_clicked ()
 		DL((GRAPP|ASSA::ASSAERR, "Unexpected response = %d, "
 			"see /path/to/gtkmm/dialog.h for details.\n", ret));
 	}
-	show ();
 }
 
 void 
@@ -544,26 +578,66 @@ on_edit_clicked ()
 		em.run ();
 		return;
 	}
+	DL ((GRAPP,"m_row_idx = %d\n", m_row_idx));
 	Gtk::TreeModel::Row row = *m_list_store_ref->children ()[m_row_idx];
 	VCard* card_iter = row [m_columns.m_card];
 
-//#ifdef IS_HILDON
-	hide ();
-//#endif
-
-	CardView cview (card_iter);
-	int ret = cview.run ();
-
-	if (ret == Gtk::RESPONSE_OK) {
-		row [m_columns.m_front_row] = 
-			Granule::trim_multiline(card_iter->get_question());
-		row [m_columns.m_back_row ] = 
-			Granule::trim_multiline(card_iter->get_answer  ());
+	if (card_iter == NULL) {
+		DL ((GRAPP, "About to crash : card_iter = NULL!\n"));
 	}
 
-//#ifdef IS_HILDON
-	show ();
-//#endif
+	CardView cview (card_iter);
+	int ret = cview.run_in_edit_mode (*this);
+
+	if (ret == Gtk::RESPONSE_OK) {
+		row [m_columns.m_front_row] = Granule::trim_multiline (
+			card_iter->get_question());
+
+		row [m_columns.m_back_row ] = Granule::trim_multiline (
+			card_iter->get_answer  ());
+	}
+}
+
+void 
+DeckView::
+on_autofill_clicked ()
+{
+	trace_with_mask("DeckView::on_autofill_clicked", GUITRACE);
+
+	CardAutoFill card_autofill_dialog;
+
+	CONFIG->autofill_db ().update_view (card_autofill_dialog);
+
+	int ret = card_autofill_dialog.run (*this);
+
+	if (ret == Gtk::RESPONSE_CANCEL) {
+		DL((GRAPP,"CANCEL clicked.\n"));
+		return;
+	}
+
+	CONFIG->autofill_db ().fetch_from_view (card_autofill_dialog);
+
+	if (ret == Gtk::RESPONSE_OK) {
+		DL((GRAPP,"OK clicked.\n"));
+	}
+	else if (ret == Gtk::RESPONSE_APPLY) {
+		DL((GRAPP,"Apply clicked.\n"));
+		if (CONFIG->autofill_db ().is_enabled ()) {
+			int count = 1;
+			VDeck::cardlist_iterator iter = m_deck.begin ();
+			while (iter != m_deck.end ()) {
+				Card* card = dynamic_cast<Card*> (*iter);
+				CONFIG->autofill_db ().apply_patterns (card, count);
+				iter++;
+				count++;
+			}
+			// TODO: Do I need to update the DeckView ???
+		}
+	}
+	else {						
+		DL((GRAPP|ASSA::ASSAERR, "Unexpected response = %d, "
+			"see /path/to/gtkmm/dialog.h for details.\n", ret));
+	}
 }
 
 void 
@@ -682,16 +756,13 @@ on_more_clicked ()
 {
 	trace_with_mask("DeckView::on_more_clicked", GUITRACE);
 
-//#ifdef IS_HILDON
-//	DeckInfo deck_info (*HILDONAPPWIN, dynamic_cast<Deck&>(m_deck));
-//	deck_info.run ();
-//	m_repaint_player = deck_info.appearance_changed ();
-//#else
 	DeckInfo deck_info (*this, dynamic_cast<Deck&>(m_deck));
-	deck_info.run ();
-	m_repaint_player = deck_info.appearance_changed ();
-//#endif
 
+	GRANULE->move_child_to_parent_position (*this, deck_info);
+	deck_info.run ();
+	GRANULE->move_parent_to_child_position (*this, deck_info);
+
+	m_repaint_player = deck_info.appearance_changed ();
 }
 
 void
@@ -841,11 +912,11 @@ on_add_card_to_cardbox ()
 	pathlist_iterator_t iter = pathlist.begin ();
 	while (iter != pathlist.end ()) 
 	{
-//		row_idx = *((*iter).get_indices ().begin ());
 		row_idx = *((*iter).begin ());
 		card_iter = m_deck.begin () + row_idx;
 		card_list.push_back (new CardRef (**card_iter, 
-										  dynamic_cast<Deck&>(m_deck)));
+										  dynamic_cast<Deck&>(m_deck),
+										  CARDBOX->sched_db ().efactor ()));
 		iter++;
 	}
 
@@ -948,26 +1019,79 @@ on_save_deck_clicked ()
 	DECKMGR->save_deck_cb ();
 }
 
-/** Called when button is pressed on any of the rows.
-	We filter out everything except for the double-click
-	which brings the CardView dialog to edit selected entry.
-
-	The rest of the events are handled by the default event
-	handler of TreeView afterwards.
-*/
+/** 
+ * Called when button is pressed on any of the rows.
+ * We filter out everything except for the double-click
+ * which brings the CardView dialog to edit selected entry.
+ *
+ * The rest of the events are handled by the default event
+ *  handler of TreeView afterwards.
+ */
 bool
 DeckView::
-on_mouse_click (GdkEventButton* event_)
+on_button_press_event (GdkEventButton* event_)
 {
-	trace_with_mask("DeckView::on_mouse_click", GUITRACE);
+	trace_with_mask("DeckView::on_button_press_event", GUITRACE);
 
-	if (event_->type == GDK_2BUTTON_PRESS
-#ifdef IS_HILDON
-		|| event_->type == GDK_Escape // Hildon round button (Cancel,Close)
-#endif
-		) 
+	DL ((GRAPP,"Key pressed: 0x%x\n", event_->type));
+
+	if (event_->type == GDK_2BUTTON_PRESS || event_->type == GDK_Return)
 	{
 		on_edit_clicked ();
 	}
+#ifdef IS_HILDON
+	else if (event_->type == GDK_Escape) // Hildon round button (Cancel,Close)
+	{
+		on_edit_clicked ();
+	}
+#endif
+
 	return false;
 }
+
+/**
+ * Return 'true' if keytap event has been handled.
+ */
+bool
+DeckView::
+on_key_tapped (GdkEventKey* event_)
+{
+	trace_with_mask("DeckView::on_key_tapped", GUITRACE);
+	bool ret = false;
+
+//	DL ((GRAPP,"Key event: 0x%x (KEY_PRESS=0x%x, ALT_MASK=0x%x\n", 
+//		 event_->type, Gdk::KEY_PRESS, Gdk::MOD1_MASK));
+//	DL ((GRAPP,"Key pressed: 0x%x\n", event_->keyval));
+
+	if (event_->type == GDK_KEY_PRESS) {
+		switch (event_->keyval) {
+		case GDK_Return:
+			on_edit_clicked ();
+			ret = true;
+			break;
+		
+		case GDK_Delete:
+			on_delete_clicked ();
+			ret = true;
+			break;
+		}
+
+		if (event_->state & Gdk::MOD1_MASK) 
+		{
+			ret = true;
+			switch (event_->keyval) 
+			{
+			case GDK_a:	on_add_clicked ();       break;
+			case GDK_e:	on_edit_clicked ();	     break;
+			case GDK_d:	on_delete_clicked ();    break;
+			case GDK_f:	on_autofill_clicked ();  break;
+			case GDK_p:	on_more_clicked ();      break;
+			case GDK_s:	on_save_deck_clicked (); break;
+				
+			default: ret = false;
+			}
+		}
+	}
+
+	return ret;
+}
