@@ -1,10 +1,10 @@
 // -*- c++ -*-
 //------------------------------------------------------------------------------
-// $Id: DeckView.h,v 1.31 2007/01/03 03:49:11 vlg Exp $
+// $Id: DeckView.h,v 1.40 2008/02/04 04:02:21 vlg Exp $
 //------------------------------------------------------------------------------
 //                            DeckView.h
 //------------------------------------------------------------------------------
-//  Copyright (c) 2004,2005 by Vladislav Grinchenko 
+//  Copyright (c) Vladislav Grinchenko 
 //
 //  This program is free software; you can redistribute it and/or 
 //  modify it under the terms of the GNU General Public License   
@@ -35,35 +35,53 @@
 #include <gtkmm/toolbar.h>
 
 #ifdef IS_HILDON
-#   include <hildon-libsmm.h>
+#   include <hildonmm.h>
 #endif
 
 class VDeck;
+class ButtonWithImageLabel;
 
+//------------------------------------------------------------------------------
+// Class DeckView
+//------------------------------------------------------------------------------
+//
 #ifdef IS_HILDON 
 class DeckView : public Hildon::Window
 #else
-class DeckView : public Gtk::Dialog
+class DeckView : public Gtk::Window
 #endif
 {
 public:
 	DeckView (VDeck& deck_, int selected_row_);
 	~DeckView ();
 
-//#ifdef IS_HILDON
-	gint run ();
-//#endif
+	/** Simulate re-entrant event loop. Also, hide/show parent
+	 *  window and reposition accordingly.
+	 */
+	gint run (Gtk::Window& parent_);
+
+	/** We block [x] delete action and reroute it to <Close>
+	 */
+	virtual bool on_delete_event (GdkEventAny* event_) 
+	{
+		on_close_clicked ();
+		return true;
+	}
 
 	/** Has the Deck been modified so that it might invalidate
-		the iterators? Deleting a card and moving card(s) up/down
-		might invalidated external iterators. 
-		Editing/Saving, however, does not.
-	*/
+	 *	the iterators? Deleting a card and moving card(s) up/down
+	 *  might invalidated external iterators. 
+	 *	Editing/Saving, however, does not.
+	 */
 	bool is_modified () const { return m_modified; }
 
+	/** Get a hold of the Deck
+	 */
+	VDeck& get_deck () const { return m_deck; }
+
 	/** Return true if user changed Appearance settings with
-		DeckInfo dialog.
-	*/
+	 *	DeckInfo dialog.
+	 */
 	bool appearance_changed () const { return m_repaint_player; }
 
 	/** User selected a row in the cards view list.
@@ -72,14 +90,14 @@ public:
 
 	/** User double-clicked on a row in the cards view list.
 	 */
-	bool on_mouse_click (GdkEventButton* event_);
+	bool on_button_press_event (GdkEventButton* event_);
 
 	/** Called in response to pressing <Close> button
 	 */
 	void on_close_clicked ();
 
 	/** Called when the user clicks on the column header to 
-		sort the list of words.
+	 *	sort the list of words.
 	 */
 	void on_column_sorted ();
 
@@ -102,6 +120,7 @@ public:
 	// -*- Sidebar action button callbacks -*-
 	void on_add_clicked         ();
 	void on_edit_clicked        ();
+	void on_autofill_clicked    ();
 	void on_delete_clicked      ();
 	void on_more_clicked        ();
 	void on_card_up_clicked     ();
@@ -109,6 +128,9 @@ public:
 	void on_add_card_to_cardbox ();
 	void on_save_deck_clicked   ();
 
+	/// [Enter] key brings up edit dialog
+	bool on_key_tapped (GdkEventKey* event_);
+
 	void add_new_card (Card* card_);
 
 	/// Return number of cards in the view
@@ -170,6 +192,7 @@ private:
 	 */
 	Gtk::Widget* m_add_button;
 	Gtk::Widget* m_edit_button;
+	Gtk::Widget* m_autofill_button;
 	Gtk::Widget* m_delete_button;
 	Gtk::Widget* m_card_up_button;
 	Gtk::Widget* m_card_down_button;
@@ -177,19 +200,16 @@ private:
 	Gtk::Widget* m_more_button;
 	Gtk::Widget* m_save_deck_button;
 
-	Gtk::Button* m_close_button;
+	ButtonWithImageLabel* m_close_button;
 
 	Gtk::Label*  m_label_count;
 
 	VDeck&       m_deck;
-	int          m_row_idx;		// The row selected in the model
-	bool         m_modified;	// Has Deck been modified to invalidate?
+	int          m_row_idx;		    // The row selected in the model
+	bool         m_modified;	    // Has Deck been modified to invalidate?
 	bool         m_repaint_player;	// Has Appearance prefs been modified?
 
-#ifdef IS_HILDON
 	Gtk::VBox*   m_vbox;
-#endif
-
 };
 
 inline int
