@@ -1,6 +1,6 @@
 // -*- c++ -*-
 //------------------------------------------------------------------------------
-// $Id: EditControls.cpp,v 1.10 2007/01/03 03:49:11 vlg Exp $
+// $Id: EditControls.cpp,v 1.13 2008/01/11 03:52:46 vlg Exp $
 //------------------------------------------------------------------------------
 //                            EditControls.cpp
 //------------------------------------------------------------------------------
@@ -24,7 +24,7 @@
 #include "Granule-main.h"
 #include "Granule.h"
 #include "EditControls.h"
-#include "MyInputDialog.h"
+#include "InputDialog.h"
 
 #include "Intern.h"
 
@@ -183,9 +183,8 @@ EditControls (Glib::RefPtr<Gtk::TextBuffer> tb_ref_)
 											  (*iter).get_stock_id (),
 											  (*iter).get_name (),
 											  (*iter).get_tooltip ()),
-						 bind<string>(mem_fun (*this, 
-										 &EditControls::on_edit_button_clicked),
-									  (*iter).get_name ()));
+		 bind<string>(mem_fun (*this, &EditControls::on_edit_button_clicked),
+					  (*iter).get_name ()));
 		iter++;
 	}
 
@@ -210,7 +209,7 @@ EditControls (Glib::RefPtr<Gtk::TextBuffer> tb_ref_)
 			"  </toolbar>"
 			"</ui>";
 
-#ifdef GLIBMM_EXCEPTOINS_ENABLED
+#ifdef GLIBMM_EXCEPTIONS_ENABLED
 		m_uimanager->add_ui_from_string (ui_info);
 #else
 		std::auto_ptr<Glib::Error> err;
@@ -484,7 +483,7 @@ add_index (Glib::RefPtr<Gtk::TextBuffer::Mark> begin_mark_,
 	Glib::ustring idx_val;
 
 	oss << m_index_count;
-	MyInputDialog ask_index (_("Enter index number:"), oss.str ());
+	InputDialog ask_index (_("Enter index number:"), oss.str ());
 	oss.str ("");
 
 	if (ask_index.run () == Gtk::RESPONSE_CANCEL) {
