@@ -2,7 +2,7 @@
 //------------------------------------------------------------------------------
 //                              GrappConf.cpp
 //------------------------------------------------------------------------------
-// $Id: GrappConf.cpp,v 1.11 2007/01/15 04:09:08 vlg Exp $
+// $Id: GrappConf.cpp,v 1.17 2007/12/28 04:06:30 vlg Exp $
 //------------------------------------------------------------------------------
 //  Copyright (c) 2004-2006 by Vladislav Grinchenko
 //
@@ -25,12 +25,6 @@
 #include <sys/types.h>			// stat(3)
 #include <sys/stat.h>			// stat(3)
 
-
-#if !defined(WIN32)
-#  include <libgen.h>			// basename(3), dirname(3)
-#  include <pwd.h>				// getpwnam(2), getlogin(2)
-#endif
-
 #include <sstream>
 using namespace std;
 
@@ -48,48 +42,6 @@ using ASSA::Utils::split_pair;
 
 ASSA_DECL_SINGLETON(GrappConf);
 
-/*------------------------------------------------------------------------------
- * Portable replacement of libgen functions - from Earnie Boyd <earnie@us...> 
- *------------------------------------------------------------------------------
- */
-#if defined(WIN32)
-
-extern char *dirname (const char *path_)
-{
-	static char path [MAX_PATH];
-
-	char* s = g_path_get_dirname (path_);
-	::strcpy (path, s);
-	g_free (s);
-
-	return path;
-}
-
-extern char *basename (const char* path_)
-{
-	static char path [MAX_PATH];
-
-	char* s = g_path_get_basename (path_);
-	::strcpy (path, s);
-	g_free (s);
-
-	return path;
-}
-
-/*---------------------------------------------------------------------------*/
-static
-char *realpath(const char *path, char *resolved_path)
-{
-    char *pszFilePart;
-
-    if (GetFullPathNameA(path, MAXPATHLEN, resolved_path, &pszFilePart)==0) {
-        return NULL;
-	}
-    return resolved_path;
-}
-#endif /* ifdef WIN32 */
-
-
 /******************************************************************************/
 void 
 SchedReview::
@@ -120,24 +72,6 @@ GrappConf::
 GrappConf () :
     m_proj_name           ("untitled.cdf"),
     m_dont_save_project   (false),
-	m_text_fgcolor        ("black"),
-	m_text_bgcolor        ("white"),
-	m_sep_color           ("pink"),
-	m_front_x_alignment   ("center"),
-	m_front_y_alignment   ("center"),
-	m_front_x_padding     ("4"),
-	m_front_y_padding     ("0"),
-	m_front_justification ("center"),
-	m_back_x_alignment    ("center"),
-	m_back_y_alignment    ("center"),
-	m_back_x_padding      ("1"),
-	m_back_y_padding      ("1"),
-	m_back_justification  ("center"),
-	m_example_x_alignment ("left"),
-	m_example_y_alignment ("top"),
-	m_example_x_padding   ("2"),
-	m_example_y_padding   ("0"),
-	m_example_justification ("left"),
 	m_mwin_geometry       (0, 0, 430, 350),
 	m_dplyr_geometry      (0, 0, 583, 350),
 	m_crdview_geometry    (0, 0, 643, 386),
@@ -164,24 +98,9 @@ GrappConf () :
     m_path_name = ptr;
     delete [] ptr;
 
-	set_question_font ("Sans 14");
-	set_answer_font   ("Sans 14");
-	set_example_font  ("Sans 14");
-	set_input_font    ("Sans 12");
-
-	/**  If fonts description is missing, you need to find out the current
-	 *	 theme's default font. Maemo default theme has overblown fonts,
-	 *	 so we scale them down to fit windows in 800x480 resolution.
-
-	 Glib::RefPtr<Gtk::Style> style = m_example.get_style ();
-	 Pango::FontDescription default_font = style->get_font ();
-	 set_app_font (default_font.to_string ());
-
-	 */
-
-#ifdef IS_HILDON
-	set_app_font ("Bitstream Vera Sans 12");
-#endif
+	text_colors_db ().init_global_defaults ();
+	fonts_db ().init_global_defaults ();
+	app_db ().init_global_defaults ();
 
 	/** Get Real User name. Retrieved from /etc/passwd, it comes in a format
 		'John Doe,,301-JOH-NDOE'.
@@ -346,41 +265,50 @@ remove_document_history (const string& fullpath_)
  *
  * [Default]
  *    filesel_path=/path/to/
+ *
  *    text_bgcolor="white"
  *    text_fgcolor="black"
  *    sep_color="pink"
+ *
  *    question_font_desc="Sans 14"
  *    answer_font_desc="Sans 14"
  *    example_font_desc="Sans 14"
  *    input_font_desc="Sans 14"
  *    app_font_desc="Bitstream Vera Sans 12"
+ *
  *    front_x_alignment="center"
  *    front_y_alignment="center"
  *    front_x_padding="4"
  *    front_y_padding="0"
  *    front_justification="center"
+ *
  *    back_x_alignment="center"
  *    back_y_alignment="center"
  *    back_x_padding="0"
  *    back_y_padding="0"
  *    back_justification="center"
+ *
  *    example_x_alignment="center"
  *    example_y_alignment="center"
  *    example_x_padding="2"
  *    example_y_padding="0"
  *    example_justification="left"
+ *
  *    mainwin_geometry=430x350
  *    cardview_geometry_height=643x386
  *    deckplayer_geometry=583x350
  *    deckplayer_keep_aspect=true
  *    deckplayer_aspect_height=3x5
+ *
  *    snd_player_cmd="sox"
  *    snd_player_agrs="%s -t ossdsp -v 1.7  -r 48000 /dev/dsp"
  *    snd_archive_path=/usr/share/WyabdcRealPeopleTTS/
+ *    auto_pronounce=false
+ *
  *    remove_duplicates=true
  *    with_relative_paths=false
- *    auto_pronounce=false
  *    history_size=5
+ *
  *    root_x=120
  *    root_y=240
  */
@@ -450,11 +378,12 @@ load_config (int secs_in_day_, int secs_in_week_)
 		DL((APP,"Section [FlipSide] was not found in \"~/.granule\".\n"));
     }
 
-    /** Load [Scheduling] section. 
-		We have to detect and convert old format:
-
-		   old:  days/weeks
-		   new:  weeks/days/hours/minutes
+    /** 
+	 * Load [Scheduling] section. 
+	 * We have to detect and convert old format:
+	 *
+	 *   old:  days/weeks
+	 *   new:  weeks/days/hours/minutes
      */
     section = m_config->find_section ("Scheduling");
 
@@ -483,6 +412,12 @@ load_config (int secs_in_day_, int secs_in_week_)
 		}
     }
 
+	/** 
+	 * Set secs in day/week.
+	 */
+	m_secs_in_day  = secs_in_day_;
+	m_secs_in_week = secs_in_week_;
+
     /************************************************************
 	 *                 Load [Default] section                   *
      ************************************************************/
@@ -496,7 +431,8 @@ load_config (int secs_in_day_, int secs_in_week_)
 	 *  If empty or non-existent, we initialize it with 
 	 *  the current working directory.
 	 */
-    m_filesel_path    = m_config->get_value ("Default", "filesel_path");
+    m_filesel_path = m_config->get_value ("Default", "filesel_path");
+
 	if (m_filesel_path.length () == 0 ||
 		Glib::path_is_absolute (m_filesel_path) == FALSE)
 	{
@@ -525,123 +461,20 @@ load_config (int secs_in_day_, int secs_in_week_)
     DL((APP,"m_snd_player_cmd   = \"%s\"\n", m_snd_player_cmd.c_str ()));
     DL((APP,"m_snd_player_args  = \"%s\"\n", m_snd_player_args.c_str ()));
     DL((APP,"m_snd_archive_path = \"%s\"\n", m_snd_archive_path.c_str ()));
+    DL((APP,"m_relative_paths   = \"%s\"\n", 
+		(m_with_relative_paths ? "true" : "false")));
 
-	ustr = m_config->get_value ("Default", "text_bgcolor");
-	if (! ustr.empty ()) {
-		text_bgcolor (ustr);
-		DL((APP,"m_text_bgcolor     = \"%s\"\n", m_text_bgcolor.c_str ()));
-	}
+	// Load text colors
+	m_text_colors_db.load_from_ini_config (m_config);
 
-	ustr = m_config->get_value ("Default", "text_fgcolor");
-	if (! ustr.empty ()) {
-		text_fgcolor (ustr);
-		DL((APP,"m_text_fgcolor     = \"%s\"\n", m_text_fgcolor.c_str ()));
-	}
-	ustr = m_config->get_value ("Default", "sep_color");
-	if (! ustr.empty ()) {
-		sep_color (ustr);
-	}
+	// Text fonts
+	m_fonts_db.load_from_ini_config (m_config);
 
-    ustr = m_config->get_value ("Default", "question_font_desc");
-    if (! ustr.empty ()) {
-		set_question_font (ustr);
-    }
-
-    ustr = m_config->get_value ("Default", "answer_font_desc");
-    if (! ustr.empty ()) {
-		set_answer_font (ustr);
-    }
-
-    ustr = m_config->get_value ("Default", "example_font_desc");
-    if (! ustr.empty ()) {
-		set_example_font (ustr);
-    }
-
-    ustr = m_config->get_value ("Default", "input_font_desc");
-    if (! ustr.empty ()) {
-		set_input_font (ustr);
-    }
-
-    ustr = m_config->get_value ("Default", "app_font_desc");
-    if (! ustr.empty ()) {
-		set_app_font (ustr);
-    }
-
-    s = m_config->get_value ("Default", "front_x_alignment");
-    if (! s.empty ()) {
-		m_front_x_alignment = s;
-    }
-
-    s = m_config->get_value ("Default", "front_y_alignment");
-    if (! s.empty ()) {
-		m_front_y_alignment = s;
-    }
-
-    s = m_config->get_value ("Default", "front_x_padding");
-    if (! s.empty ()) {
-		m_front_x_padding = s;
-    }
-
-    s = m_config->get_value ("Default", "front_y_padding");
-    if (! s.empty ()) {
-		m_front_y_padding = s;
-    }
-
-    s = m_config->get_value ("Default", "front_justification");
-    if (! s.empty ()) {
-		m_front_justification = s;
-    }
-
-    s = m_config->get_value ("Default", "back_x_alignment");
-    if (! s.empty ()) {
-		m_back_x_alignment = s;
-    }
-
-    s = m_config->get_value ("Default", "back_y_alignment");
-    if (! s.empty ()) {
-		m_back_y_alignment = s;
-    }
-
-    s = m_config->get_value ("Default", "back_x_padding");
-    if (! s.empty ()) {
-		m_back_x_padding = s;
-    }
-
-    s = m_config->get_value ("Default", "back_y_padding");
-    if (! s.empty ()) {
-		m_back_y_padding = s;
-    }
-
-    s = m_config->get_value ("Default", "back_justification");
-    if (! s.empty ()) {
-		m_back_justification = s;
-    }
-
-    s = m_config->get_value ("Default", "example_x_alignment");
-    if (! s.empty ()) {
-		m_example_x_alignment = s;
-    }
-
-    s = m_config->get_value ("Default", "example_y_alignment");
-    if (! s.empty ()) {
-		m_example_y_alignment = s;
-    }
-
-    s = m_config->get_value ("Default", "example_x_padding");
-    if (! s.empty ()) {
-		m_example_x_padding = s;
-    }
-
-    s = m_config->get_value ("Default", "example_y_padding");
-    if (! s.empty ()) {
-		m_example_y_padding = s;
-    }
-
-    s = m_config->get_value ("Default", "example_justification");
-    if (! s.empty ()) {
-		m_example_justification = s;
-    }
+	// Load all appearance settings
+	m_appearance.load_from_ini_config (m_config);
 
+	/** Load geometry
+	 */
     s = m_config->get_value ("Default", "mainwin_geometry");
     if (! s.empty ()) {
 		str_to_geometry (s, m_mwin_geometry);
@@ -687,9 +520,8 @@ load_config (int secs_in_day_, int secs_in_week_)
 }
 
 /******************************************************************************
- * Save configuration to the user's configuration file, ~/.granule.
- *
- */
+ * Save configuration to the user's configuration file, ~/.granule.           *
+ ******************************************************************************/
 void
 GrappConf::
 save_config ()
@@ -747,7 +579,7 @@ save_config ()
     }
     /** Save Scheduling configuration
      */
-    for (int j = 0; j < CARD_BOX_SIZE; j++) {
+    for (int j = 0; j < CARDBOXES_MIN; j++) {
 		ckey.str ("");
 		ckey << (j + 1);
 		m_config->set_pair ("Scheduling", 
@@ -780,81 +612,17 @@ save_config ()
 		iter++, position++;
     }
 
-    Glib::ustring ustr;
     m_config->set_pair ("Default", 
 						IniFile::tuple_type("filesel_path", m_filesel_path));
-	/** Text colors
-	 */
-    m_config->set_pair ("Default", 
-						IniFile::tuple_type("text_fgcolor",	m_text_fgcolor));
-    m_config->set_pair ("Default", 
-						IniFile::tuple_type("text_bgcolor",	m_text_bgcolor));
-    m_config->set_pair ("Default", 
-						IniFile::tuple_type("sep_color",	m_sep_color));
-    /** Fonts
-     */
-    ustr = m_question_font_desc.to_string ();
-    m_config->set_pair ("Default", 
-						IniFile::tuple_type("question_font_desc", ustr));
-    ustr = m_answer_font_desc.to_string ();
-    m_config->set_pair ("Default", 
-						IniFile::tuple_type("answer_font_desc", ustr));
-    ustr = m_example_font_desc.to_string ();
-    m_config->set_pair ("Default", 
-						IniFile::tuple_type("example_font_desc", ustr));
-    ustr = m_input_font_desc.to_string ();
-    m_config->set_pair ("Default", 
-						IniFile::tuple_type("input_font_desc", ustr));
-    ustr = m_app_font_desc.to_string ();
-    m_config->set_pair ("Default", 
-						IniFile::tuple_type("app_font_desc", ustr));
-	/** Text alignment
-	 */
-    m_config->set_pair ("Default", 
-						IniFile::tuple_type("front_x_alignment",	
-											m_front_x_alignment));
-    m_config->set_pair ("Default", 
-						IniFile::tuple_type("front_y_alignment",	
-											m_front_y_alignment));
-    m_config->set_pair ("Default", 
-						IniFile::tuple_type("front_x_padding",	
-											m_front_x_padding));
-    m_config->set_pair ("Default", 
-						IniFile::tuple_type("front_y_padding",	
-											m_front_y_padding));
-    m_config->set_pair ("Default", 
-						IniFile::tuple_type("front_justification",	
-											m_front_justification));
-    m_config->set_pair ("Default", 
-						IniFile::tuple_type("back_x_alignment",	
-											m_back_x_alignment));
-    m_config->set_pair ("Default", 
-						IniFile::tuple_type("back_y_alignment",	
-											m_back_y_alignment));
-    m_config->set_pair ("Default", 
-						IniFile::tuple_type("back_x_padding",	
-											m_back_x_padding));
-    m_config->set_pair ("Default", 
-						IniFile::tuple_type("back_y_padding",	
-											m_back_y_padding));
-    m_config->set_pair ("Default", 
-						IniFile::tuple_type("back_justification",	
-											m_back_justification));
-    m_config->set_pair ("Default", 
-						IniFile::tuple_type("example_x_alignment",	
-											m_example_x_alignment));
-    m_config->set_pair ("Default", 
-						IniFile::tuple_type("example_y_alignment",	
-											m_example_y_alignment));
-    m_config->set_pair ("Default", 
-						IniFile::tuple_type("example_x_padding",	
-											m_example_x_padding));
-    m_config->set_pair ("Default", 
-						IniFile::tuple_type("example_y_padding",	
-											m_example_y_padding));
-    m_config->set_pair ("Default", 
-						IniFile::tuple_type("example_justification",	
-											m_example_justification));
+	// Text colors
+	m_text_colors_db.save_to_ini_config (m_config);
+
+    // Text Fonts
+	m_fonts_db.save_to_ini_config (m_config);
+
+	// Text alignment
+	m_appearance.save_to_ini_config (m_config);
+
     /** Geometry
      */
     m_config->set_pair ("Default", 
@@ -966,30 +734,14 @@ set_flipside_history (const string& fname_, SideSelection side_)
 	}
 }
 
-/** 
- * Given the full file path, split it into dirname and basename.
- *
- * From man dirname(3):
- *
- *   "Both dirname and basename may modify the contents of path,
- *    so if you need to  preserve  the  pathname string,
- *    copies should be passed to these functions.
- *
- *    Furthermore, dirname and basename may return  pointers  to
- *    statically  allocated  memory which may overwritten by subsequent
- *     calls."
- */
 void
 GrappConf::
 set_proj_name (const string& fname_)
 {
-    char s [PATH_MAX];
-    strcpy (s, fname_.c_str ());
-    m_proj_name = basename (s); // 's' might have been modified by basename() !
+    m_proj_name = GRANULE->get_basename (fname_.c_str ());
+    m_path_name = GRANULE->get_dirname  (fname_.c_str ());
+
 	DL ((GRAPP, "New project name: \"%s\"\n", m_proj_name.c_str ()));
-                                                                                
-    strcpy (s, fname_.c_str ());
-    m_path_name = dirname  (s);
 	DL ((GRAPP, "New project path: \"%s\"\n", m_path_name.c_str ()));
 }
 
