@@ -2,7 +2,7 @@
 //------------------------------------------------------------------------------
 //                              GrappConf.h
 //------------------------------------------------------------------------------
-// $Id: GrappConf.h,v 1.9 2007/01/07 20:16:43 vlg Exp $
+// $Id: GrappConf.h,v 1.17 2007/12/28 04:06:30 vlg Exp $
 //------------------------------------------------------------------------------
 //  Copyright (c) 2004-2006 by Vladislav Grinchenko
 //
@@ -24,7 +24,6 @@ using std::string;
 #include <pangomm/fontdescription.h>
 
 #include <gdkmm/rectangle.h>
-#include <gdkmm/color.h>
 
 #include <assa/Singleton.h>
 #include <assa/TimeVal.h>
@@ -33,8 +32,14 @@ class ASSA::IniFile;
 
 #include "Granule-main.h"
 
+#include "AppearanceDB.h"
+#include "FontsDB.h"
+#include "TextColorsDB.h"
+#include "AutoFillDB.h"
+
+//------------------------------------------------------------------------------
 // Record when to review cards in box I (I=DAYS/WEEKS)
-//
+//------------------------------------------------------------------------------
 struct SchedReview 
 {
     string m_days;			// number of days
@@ -63,6 +68,9 @@ struct SchedReview
     void   dump (int idx_);
 };
 
+//------------------------------------------------------------------------------
+// GrappConf - persistent storage via INI file.
+//------------------------------------------------------------------------------
 class GrappConf : public ASSA::Singleton<GrappConf>
 {
 public:
@@ -99,11 +107,11 @@ public:
     void with_relpaths (bool b_) { m_with_relative_paths = b_;   }
 
     /** Returns an expiration date that should be assigned to a 
-	card as it is placed in CardDeck # idx_. 
-
-	@param idx_ CardDeck number.
-	@return Card expiration time.
-    */
+	 *	card as it is placed in CardDeck # idx_. 
+	 *
+	 *  @param idx_ CardDeck number.
+	 *	@return Card expiration time.
+	 */
     ASSA::TimeVal get_expiration (int idx_);
 
     /** Access to the scheduling settings by index.
@@ -118,6 +126,8 @@ public:
     void set_sched_review_hours (int idx_, const string& s_);
     void set_sched_review_mins  (int idx_, const string& s_);
 
+	long secs_in_day () const { return m_secs_in_day; }
+
     /// Return timestamp when the program was started.
     ASSA::TimeVal get_load_time () const { return m_load_time; }
 
@@ -131,41 +141,19 @@ public:
 
 	/** Text colors
 	 */
-	Gdk::Color text_fgcolor () const { return Gdk::Color (m_text_fgcolor); }
-	Gdk::Color text_bgcolor () const { return Gdk::Color (m_text_bgcolor); }
-	Gdk::Color sep_color    () const { return Gdk::Color (m_sep_color);    }
-
-	void text_fgcolor (const Glib::ustring& s_) { m_text_fgcolor = s_; }
-	void text_bgcolor (const Glib::ustring& s_) { m_text_bgcolor = s_; }
-	void sep_color    (const Glib::ustring& s_) { m_sep_color    = s_; }
+	TextColorsDB& text_colors_db () { return m_text_colors_db; }
 
     /** Text fonts settings
      */
-    Pango::FontDescription& get_question_font () { return m_question_font_desc;}
-    Pango::FontDescription& get_answer_font   () { return m_answer_font_desc;  }
-    Pango::FontDescription& get_example_font  () { return m_example_font_desc; }
-    Pango::FontDescription& get_input_font    () { return m_input_font_desc;   }
-    Pango::FontDescription& get_app_font      () { return m_app_font_desc;     }
-
-    void set_question_font (const Glib::ustring& font_);
-    void set_answer_font   (const Glib::ustring& font_);
-    void set_example_font  (const Glib::ustring& font_);
-    void set_input_font    (const Glib::ustring& font_);
-    void set_app_font      (const Glib::ustring& font_);
+	FontsDB& fonts_db () { return m_fonts_db; }
 
 	/** Text alignment controls
 	 */
-	string x_alignment   (SideSelection s_) const;
-	string y_alignment   (SideSelection s_) const;
-	string x_padding     (SideSelection s_) const;
-	string y_padding     (SideSelection s_) const;
-	string justification (SideSelection s_) const;
+	AppearanceDB& app_db () { return m_appearance; }
 
-	void x_alignment   (SideSelection s_, const string& v_);
-	void y_alignment   (SideSelection s_, const string& v_);
-	void x_padding     (SideSelection s_, const string& v_);
-	void y_padding     (SideSelection s_, const string& v_);
-	void justification (SideSelection s_, const string& v_);
+	/** Card AutoFill database is not archived, still hosted here
+	 */
+	AutoFillDB& autofill_db () { return m_autofill_db; }
 
     /** Windows geometry
      */
@@ -240,34 +228,24 @@ private:
     string m_filesel_path;		// Path of the last FileSelection dialog
     bool   m_dont_save_project;	        // Don't save project file
 
-	Glib::ustring m_text_fgcolor;
-	Glib::ustring m_text_bgcolor;
-	Glib::ustring m_sep_color;
+	/** Text colors database is the data model.
+	 */
+	TextColorsDB m_text_colors_db;
 
-    Pango::FontDescription m_question_font_desc;
-    Pango::FontDescription m_answer_font_desc;
-    Pango::FontDescription m_example_font_desc;
-    Pango::FontDescription m_input_font_desc;
-    Pango::FontDescription m_app_font_desc;
+	/** Fonts database is the data model.
+	 */
+	FontsDB m_fonts_db;
 
-	string m_front_x_alignment;
-	string m_front_y_alignment; 
-	string m_front_x_padding;
-	string m_front_y_padding; 
-	string m_front_justification; 
+	/** Text appearance database is the data model.
+	 */
+	AppearanceDB m_appearance;
 
-	string m_back_x_alignment;
-	string m_back_y_alignment; 
-	string m_back_x_padding;
-	string m_back_y_padding; 
-	string m_back_justification; 
-
-	string m_example_x_alignment;
-	string m_example_y_alignment; 
-	string m_example_x_padding;
-	string m_example_y_padding; 
-	string m_example_justification; 
+	/** Card AutoFill database is the data model.
+	 */
+	AutoFillDB m_autofill_db;
 
+	/** All things geometry
+	 */
     Gdk::Rectangle  m_mwin_geometry;
     Gdk::Rectangle  m_dplyr_geometry;
     Gdk::Rectangle  m_crdview_geometry;
@@ -275,6 +253,8 @@ private:
     bool m_dplyr_keep_aspect;
     Gdk::Rectangle  m_dplyr_aspect; 
 
+	/** Sound-related settings
+	 */
     string m_snd_player_cmd;	        // Sound player string
     string m_snd_player_args;           // Sound player arguments formatted 
                                         // string.
@@ -283,16 +263,21 @@ private:
 
     DHList         m_history;	        // CardFile history list.
     int            m_history_size;      // Maximum size of the history list.
+
     ASSA::IniFile* m_config;
+
     flipside_t     m_flipside_history;	// Which side to show for each Deck.
     bool           m_remove_dups;       // Purge CardDeck of duplicate VCards
     bool           m_with_relative_paths; // Record relative paths to the Decks.
+
     ASSA::TimeVal  m_load_time;	        // Time the program has been loaded
-    SchedReview    m_sched [CARD_BOX_SIZE];	// Review schedule
-    int            m_secs_in_day;       // Seconds in a day
-    int            m_secs_in_week;      // Seconds in a week
+    SchedReview    m_sched [CARDBOXES_MIN];	// Review schedule
+    long           m_secs_in_day;       // Seconds in a day
+    long           m_secs_in_week;      // Seconds in a week
+
     int            m_root_x;	        // X-coordinate of MainWindow on desktop
     int            m_root_y;            // Y-coordinate of MainWindow on desktop
+
     bool           m_disable_key_shortcuts; // No keyboard navigation
     bool           m_auto_pronounce;    // Auto-pronounce each card
 };
@@ -306,38 +291,6 @@ private:
 //------------------------------------------------------------------------------
 //
 
-/** Text fonts
- */
-inline void 
-GrappConf::set_question_font (const Glib::ustring& font_)
-{
-    m_question_font_desc = Pango::FontDescription (font_);
-}
-
-inline void 
-GrappConf::set_answer_font (const Glib::ustring& font_)
-{
-    m_answer_font_desc = Pango::FontDescription (font_);
-}
-
-inline void 
-GrappConf::set_example_font (const Glib::ustring& font_)
-{
-    m_example_font_desc = Pango::FontDescription (font_);
-}
-
-inline void 
-GrappConf::set_input_font (const Glib::ustring& font_)
-{
-    m_input_font_desc = Pango::FontDescription (font_);
-}
-
-inline void 
-GrappConf::set_app_font (const Glib::ustring& font_)
-{
-    m_app_font_desc = Pango::FontDescription (font_);
-}
-
 /** Windows geometry
  */
 inline void 
@@ -428,94 +381,5 @@ GrappConf::set_sched_review_mins (int idx_, const string& s_)
 { 
     m_sched [idx_].mins (s_); 
 }
-
-/**-------------------------
- **** Retrieve a property **
- **-------------------------
- */
-inline string 
-GrappConf::x_alignment (SideSelection s_) const 
-{ 
-	if      (s_ == FRONT) { return m_front_x_alignment;   }
-	else if (s_ == BACK)  { return m_back_x_alignment;    }
-	else    /* EXAMPLE */ { return m_example_x_alignment; }
-}
-
-inline string
-GrappConf::y_alignment (SideSelection s_) const
-{ 
-	if      (s_ == FRONT) { return m_front_y_alignment;   }
-	else if (s_ == BACK)  { return m_back_y_alignment;    }
-	else    /* EXAMPLE */ { return m_example_y_alignment; }
-}
-
-inline string 
-GrappConf::x_padding (SideSelection s_) const 
-{ 
-	if      (s_ == FRONT) { return m_front_x_padding;   }
-	else if (s_ == BACK)  { return m_back_x_padding;    }
-	else    /* EXAMPLE */ { return m_example_x_padding; }
-}
-
-inline string
-GrappConf::y_padding (SideSelection s_) const
-{ 
-	if      (s_ == FRONT) { return m_front_y_padding;   }
-	else if (s_ == BACK)  { return m_back_y_padding;    }
-	else    /* EXAMPLE */ { return m_example_y_padding; }
-}
-
-inline string
-GrappConf::justification (SideSelection s_) const 
-{ 
-	if      (s_ == FRONT) { return m_front_justification;   }
-	else if (s_ == BACK)  { return m_back_justification;    }
-	else    /* EXAMPLE */ { return m_example_justification; }
-}
-
-/**--------------------
- **** Set a property **
- **--------------------
- */
-inline void
-GrappConf::x_alignment (SideSelection s_, const string& v_)
-{ 
-	if      (s_ == FRONT) { m_front_x_alignment   = v_; }
-	else if (s_ == BACK)  { m_back_x_alignment    = v_; }
-	else    /* EXAMPLE */ { m_example_x_alignment = v_; }
-}
-
-inline void
-GrappConf::y_alignment (SideSelection s_, const string& v_)
-{ 
-	if      (s_ == FRONT) { m_front_y_alignment   = v_; }
-	else if (s_ == BACK)  { m_back_y_alignment    = v_; }
-	else    /* EXAMPLE */ { m_example_y_alignment = v_; }
-}
-
-inline void
-GrappConf::x_padding (SideSelection s_, const string& v_)
-{ 
-	if      (s_ == FRONT) { m_front_x_padding   = v_; }
-	else if (s_ == BACK)  { m_back_x_padding    = v_; }
-	else    /* EXAMPLE */ { m_example_x_padding = v_; }
-}
-
-inline void
-GrappConf::y_padding (SideSelection s_, const string& v_)
-{ 
-	if      (s_ == FRONT) { m_front_y_padding   = v_; }
-	else if (s_ == BACK)  { m_back_y_padding    = v_; }
-	else    /* EXAMPLE */ { m_example_y_padding = v_; }
-}
-
-inline void
-GrappConf::justification (SideSelection s_, const string& v_)
-{ 
-	if      (s_ == FRONT) { m_front_justification   = v_; }
-	else if (s_ == BACK)  { m_back_justification    = v_; }
-	else    /* EXAMPLE */ { m_example_justification = v_; }
-}
-
     
 #endif /* CONFIG_H */
