@@ -1,6 +1,6 @@
 // -*- c++ -*-
 //------------------------------------------------------------------------------
-// $Id: PrefWindow.cpp,v 1.13 2006/11/05 19:00:59 vlg Exp $
+// $Id: PrefWindow.cpp,v 1.16 2008/01/06 16:51:10 vlg Exp $
 //------------------------------------------------------------------------------
 //                            PrefWindow.h
 //------------------------------------------------------------------------------
@@ -22,7 +22,7 @@
 PrefWindow::
 PrefWindow (Gtk::Widget& parent_) 
 	:
-    MyPropertyBox      ("Preferences"),
+    PropertyBox      ("Preferences"),
 	m_general_pref     (*this),
 	m_appearance_pref  (*this),
 	m_scheduling_pref  (*this),
@@ -52,7 +52,7 @@ PrefWindow (Gtk::Widget& parent_)
 	 */
 	append_page (m_general_pref,    "General"   );
 	append_page (m_appearance_pref, "Appearance");
-	append_page (m_scheduling_pref, "Scheduling");
+//	append_page (m_scheduling_pref, "Scheduling");
 	append_page (m_sound_pref,      "Sound"     );
 	append_page (m_geometry_pref,   "Geometry"  );
 }
@@ -70,7 +70,7 @@ run ()
 	 */
 	m_general_pref.   load_from_config ();
 	m_appearance_pref.load_from_config ();
-	m_scheduling_pref.load_from_config ();
+//	m_scheduling_pref.load_from_config ();
 	m_sound_pref.     load_from_config ();
 	m_geometry_pref.  load_from_config ();
 	
@@ -91,7 +91,7 @@ apply_impl ()
 
 	m_general_pref.   save_to_config ();
 	m_appearance_pref.save_to_config ();
-	m_scheduling_pref.save_to_config ();
+//	m_scheduling_pref.save_to_config ();
 	m_sound_pref.     save_to_config ();
 	m_geometry_pref.  save_to_config ();
 }
