@@ -1,6 +1,6 @@
 // -*- c++ -*-
 //------------------------------------------------------------------------------
-// $Id: PrefWindow.h,v 1.11 2006/09/03 19:26:36 vlg Exp $
+// $Id: PrefWindow.h,v 1.13 2008/02/04 04:02:21 vlg Exp $
 //------------------------------------------------------------------------------
 //                            PrefWindow.h
 //------------------------------------------------------------------------------
@@ -22,14 +22,14 @@
 
 #include "Granule-main.h"
 #include "GrappConf.h"
-#include "MyPropertyBox.h"
+#include "PropertyBox.h"
 #include "GeneralPref.h"
 #include "SchedulingPrefs.h"
 #include "SoundPrefs.h"
 #include "GeometryPrefs.h"
 #include "AppearancePrefs.h"
 
-class PrefWindow : public MyPropertyBox
+class PrefWindow : public PropertyBox
 {
 public:
     PrefWindow (Gtk::Widget& parent_);
