@@ -1,6 +1,6 @@
 // -*- c++ -*-
 //------------------------------------------------------------------------------
-// $Id: TextAlignment.h,v 1.5 2006/12/18 03:33:15 vlg Exp $
+// $Id: TextAlignment.h,v 1.7 2007/12/02 00:31:27 vlg Exp $
 //------------------------------------------------------------------------------
 //                            TextAlignment.h
 //------------------------------------------------------------------------------
@@ -19,32 +19,25 @@
 #define TEXT_ALIGNMENT_H
 
 #include "PrefWindow.h"
-#include "MyTextAlignmentsWidget.h"
+#include "TextAlignmentsUI.h"
 
 class TextAlignment : public Gtk::VBox
 {
 public:
 	TextAlignment (PrefWindow& grandpa_);
 
-	void set_x_alignment   (SideSelection side_, const std::string& v_);
-	void set_y_alignment   (SideSelection side_, const std::string& v_);
-	void set_x_padding     (SideSelection side_, const std::string& v_);
-	void set_y_padding     (SideSelection side_, const std::string& v_);
-	void set_justification (SideSelection side_, const std::string& v_);
-
-	std::string get_x_alignment   (SideSelection side_) const;
-	std::string get_y_alignment   (SideSelection side_) const;
-	std::string get_x_padding     (SideSelection side_) const;
-	std::string get_y_padding     (SideSelection side_) const;
-	std::string get_justification (SideSelection side_) const;
+	/** Sync appearance settings with GrappConf persistent storage.
+	 */
+    void load_appearance (AppearanceDB& app_db_);
+    void save_appearance (AppearanceDB& app_db_);
 
 private:
-	PrefWindow&      m_pref_window;
-	Gtk::VBox*       m_vbox;			/* parent */
+	PrefWindow& m_pref_window;
+	Gtk::VBox*  m_vbox;			/* parent */
 
-	MyTextAlignmentsWidget* m_front_widget;
-	MyTextAlignmentsWidget* m_back_widget;
-	MyTextAlignmentsWidget* m_example_widget;
+	TextAlignmentsUI* m_front_widget;
+	TextAlignmentsUI* m_back_widget;
+	TextAlignmentsUI* m_example_widget;
 };
 
 #endif /* TEXT_ALIGNMENT_H */
