@@ -2,9 +2,9 @@
 //------------------------------------------------------------------------------
 //                              TopMenuBar.h
 //------------------------------------------------------------------------------
-// $Id: TopMenuBar.h,v 1.17 2006/11/14 03:33:27 vlg Exp $
+// $Id: TopMenuBar.h,v 1.18 2007/12/26 03:32:23 vlg Exp $
 //------------------------------------------------------------------------------
-//  Copyright (c) 2004,2006 by Vladislav Grinchenko
+//  Copyright (c) 2004,2006-2007 by Vladislav Grinchenko
 //
 //  This program is free software; you can redistribute it and/or
 //  modify it under the terms of the GNU General Public License
@@ -31,6 +31,11 @@ class DeckManager;
 using namespace Gtk;
 using namespace Menu_Helpers;
 
+/**
+ * @class TopMenuBar
+ *
+ * This class holds the top menues.
+ */
 class TopMenuBar : public sigc::trackable
 {
 public:
@@ -84,6 +89,7 @@ private:
     MenuItem*    m_export;
     MenuItem*    m_close;
     MenuItem*    m_play_in;
+	MenuItem*    m_cb_preferences;  // CardBox Preferences
 
 	// -*- [Deck] items -*-
 	//
